#include "StdInc.h"
#include "RollEm.h"

#include "RollTypes.h"

DieRoll g_drPaste;
DieRoll g_drCustom;

MemHandle hTitle;
MemHandle hNumber; 
MemHandle hDie;
MemHandle hModifier;
MemHandle hBest;
MemHandle hCompare1;
MemHandle hCompare2;
MemHandle hSuccess;

Boolean CustomProc (EventPtr pEvent);
void InitializeCustom (FormPtr pForm);
void CopyCustomRoll (DieRollPtr dieRollPtr);
void PasteCustomRoll (DieRollPtr dieRollPtr);

void DoModalCustom (DieRollPtr pDieRoll)
{
	FormPtr pForm, pOldForm;
	UInt16 wResult;

	wResult = 0;
	pOldForm = FrmGetActiveForm();
	if (pForm = FrmInitForm(IDF_CUSTOM))
	{
		// Pre form stuff.
		FrmSetActiveForm(pForm);
		FrmDrawForm(pForm);

		InitializeCustom(pForm);
		MemMove(&g_drCustom, pDieRoll, sizeof(DieRoll));
		PasteCustomRoll(&g_drCustom);
		FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_TITLE));

		FrmSetEventHandler(pForm, (FormEventHandlerPtr) CustomProc);
		wResult = FrmDoDialog(pForm);

		if (wResult == IDC_OK)
		{
			MemMove(pDieRoll, &g_drCustom, sizeof(DieRoll));
		}

		// We now return to our other window.
		if (pOldForm)
			FrmSetActiveForm(pOldForm);
		FrmDeleteForm(pForm);
	}
}

Boolean CustomProc (EventPtr pEvent)
{
	FormPtr pForm = FrmGetActiveForm();
	Boolean bHandled = false;

    switch (pEvent->eType) 
    {
	case frmUpdateEvent:
	{
		FrmDrawForm(pForm);
		bHandled = true;
		break;
	}

    case ctlSelectEvent:  // A control button was pressed and released.
	{
        switch (pEvent->data.ctlEnter.controlID)
        {
        case IDC_OK:
        {
            Char szMinRange [10], szMaxRange [10];

			CopyCustomRoll(&g_drCustom);
            
            // Validate all the data
            if ((g_drCustom.lField[NORMAL_NUMBER] < MIN_NUMBER) || 
                (g_drCustom.lField[NORMAL_NUMBER] > MAX_NUMBER))
            {
                StrLToA(szMinRange, (Int32)MIN_NUMBER);
                StrLToA(szMaxRange, (Int32)MAX_NUMBER);
                FrmCustomAlert(IDA_RANGE_ERROR, "number of dice", szMinRange, szMaxRange);
                FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_NUMBER));
				bHandled = true;
                break;
            }
            if ((g_drCustom.lField[NORMAL_DIE] < MIN_DIE) || 
                (g_drCustom.lField[NORMAL_DIE] > MAX_DIE))
            {
                StrLToA(szMinRange, (Int32)MIN_DIE);
                StrLToA(szMaxRange, (Int32)MAX_DIE);
                FrmCustomAlert(IDA_RANGE_ERROR, "die", szMinRange, szMaxRange);
                FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_DIE));
				bHandled = true;
                break;
            }
            if ((g_drCustom.lField[NORMAL_MODIFIER] < MIN_MODIFIER) || 
                (g_drCustom.lField[NORMAL_MODIFIER] > MAX_MODIFIER))
            {
                StrLToA(szMinRange, (Int32)MIN_MODIFIER);
                StrLToA(szMaxRange, (Int32)MAX_MODIFIER);
                FrmCustomAlert(IDA_RANGE_ERROR, "modifer", szMinRange, szMaxRange);
                FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_MODIFIER));
				bHandled = true;
                break;
            }
            if ((g_drCustom.lField[NORMAL_BEST] < MIN_BEST) || 
                (g_drCustom.lField[NORMAL_BEST] > MAX_BEST))
            {
                StrLToA(szMinRange, (Int32)MIN_BEST);
                StrLToA(szMaxRange, (Int32)MAX_BEST);
                FrmCustomAlert(IDA_RANGE_ERROR, "best", szMinRange, szMaxRange);
                FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_BEST));
				bHandled = true;
                break;
            }
            if ((g_drCustom.lField[COMMON_BEEPVALUE1] < MIN_COMPARE) || 
                (g_drCustom.lField[COMMON_BEEPVALUE1] > MAX_COMPARE))
            {
				StrLToA(szMinRange, (Int32)MIN_COMPARE);
				StrLToA(szMaxRange, (Int32)MAX_COMPARE);
				FrmCustomAlert(IDA_RANGE_ERROR, "compare", szMinRange, szMaxRange);
				FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_COMPARE_VALUE1));
				bHandled = true;
				break;
            }
            if ((g_drCustom.lField[COMMON_BEEPVALUE2] < MIN_COMPARE) || 
                (g_drCustom.lField[COMMON_BEEPVALUE2] > MAX_COMPARE))
            {
				StrLToA(szMinRange, (Int32)MIN_COMPARE);
				StrLToA(szMaxRange, (Int32)MAX_COMPARE);
				FrmCustomAlert(IDA_RANGE_ERROR, "compare", szMinRange, szMaxRange);
				FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_COMPARE_VALUE2));
				bHandled = true;
				break;
            }
            if ((g_drCustom.lField[SUCCESS_VALUE] < MIN_SUCCESS) || 
                (g_drCustom.lField[SUCCESS_VALUE] > MAX_SUCCESS))
            {
				StrLToA(szMinRange, (Int32)MIN_SUCCESS);
				StrLToA(szMaxRange, (Int32)MAX_SUCCESS);
				FrmCustomAlert(IDA_RANGE_ERROR, "success", szMinRange, szMaxRange);
				FrmSetFocus(pForm, FrmGetObjectIndex(pForm, IDC_SUCCESS_VALUE));
				bHandled = true;
				break;
            }
            if (StrLen(g_drCustom.szTitle) == 0)
            {
				CreateTitle(&g_drCustom, g_drCustom.szTitle, MAX_TITLE_LEN+1);
            }
        }
        case IDC_CANCEL:
			bHandled = false;
            break;
        }
        break;
	}

	case popSelectEvent:
	{
		switch(pEvent->data.popSelect.listID)
		{
        case IDC_BEEP_TYPE1:
			g_drCustom.lField[COMMON_BEEPTYPE1] = pEvent->data.popSelect.selection;
			break;
        case IDC_BEEP_TYPE2:
			g_drCustom.lField[COMMON_BEEPTYPE2] = pEvent->data.popSelect.selection;
			break;
        case IDC_COMPARE_TYPE1:
            g_drCustom.lField[COMMON_BEEPCOMPARE1] = pEvent->data.popSelect.selection;
            break;
        case IDC_COMPARE_TYPE2:
            g_drCustom.lField[COMMON_BEEPCOMPARE2] = pEvent->data.popSelect.selection;
            break;
        case IDC_SUCCESS_TYPE:
            g_drCustom.lField[SUCCESS_COMPARE] = pEvent->data.popSelect.selection;
            break;
		}
		break;
	}

    case menuEvent:
	{
        switch (pEvent->data.menu.itemID)
        {
        case IDM_EDIT_KEYBOARD:
            SysKeyboardDialog(kbdDefault);
            break;
        case IDM_EDIT_COPY_ROLL:
            CopyCustomRoll(&g_drPaste);
            break;
        case IDM_EDIT_PASTE_ROLL:
            PasteCustomRoll(&g_drPaste);
			RefreshForm();
            break;
        }
        bHandled = true;
        break;
	}

	default:
		bHandled = false;
    }

    return bHandled;
}

void InitializeCustom (FormPtr pForm)
{
	static Boolean bInit = false;

	// Do the one time initialization 
	//  of the custom dialog here.
	if (!bInit)
	{
	    MemSet(&g_drPaste, sizeof(DieRoll), '\0');
		bInit = true;
	}

	// Do the every time initialization 
	//  of the custom dialog here.

	AutoShiftField(pForm, IDC_TITLE, true);
}

void CopyCustomRoll (DieRollPtr dieRollPtr)
{
    StrCopy(dieRollPtr->szTitle, (Char*)MemHandleLock(hTitle));
    MemHandleUnlock(hTitle);
    
    dieRollPtr->lField[NORMAL_NUMBER] = StrAToI((Char*)MemHandleLock(hNumber));
    MemHandleUnlock(hNumber);
    
    dieRollPtr->lField[NORMAL_DIE] = StrAToI((Char*)MemHandleLock(hDie));
    MemHandleUnlock(hDie);
    
    dieRollPtr->lField[NORMAL_MODIFIER] = StrAToI((Char*)MemHandleLock(hModifier));
    MemHandleUnlock(hModifier);
    
    dieRollPtr->lField[NORMAL_BEST] = StrAToI((Char*)MemHandleLock(hBest));
    MemHandleUnlock(hBest);
    
    dieRollPtr->lField[COMMON_BEEPVALUE1] = (Int32)StrAToL((Char*)MemHandleLock(hCompare1));
    MemHandleUnlock(hCompare1);

    dieRollPtr->lField[COMMON_BEEPVALUE2] = (Int32)StrAToL((Char*)MemHandleLock(hCompare2));
    MemHandleUnlock(hCompare2);

    dieRollPtr->lField[SUCCESS_VALUE] = (Int32)StrAToL((Char*)MemHandleLock(hSuccess));
    MemHandleUnlock(hSuccess);
}

void PasteCustomRoll (DieRollPtr dieRollPtr)
{
	FormPtr pForm = FrmGetActiveForm();
	Char szTemp [32];
	int i;

	hTitle = SetField (pForm, IDC_TITLE, MAX_TITLE_LEN, dieRollPtr->szTitle);
	hNumber = SetField (pForm, IDC_NUMBER, MAX_NUMBER_LEN, 
		StrLToA(szTemp, (Int32)dieRollPtr->lField[NORMAL_NUMBER]));
	hDie = SetField (pForm, IDC_DIE, MAX_DIE_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[NORMAL_DIE]));
	hModifier = SetField (pForm, IDC_MODIFIER, MAX_MODIFIER_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[NORMAL_MODIFIER]));
	hBest = SetField (pForm, IDC_BEST, MAX_BEST_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[NORMAL_BEST]));

	hCompare1 = SetField (pForm, IDC_COMPARE_VALUE1, MAX_COMPARE_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[COMMON_BEEPVALUE1]));
	hCompare2 = SetField (pForm, IDC_COMPARE_VALUE2, MAX_COMPARE_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[COMMON_BEEPVALUE2]));
	hSuccess = SetField (pForm, IDC_SUCCESS_VALUE, MAX_SUCCESS_LEN,
		StrLToA(szTemp, (Int32)dieRollPtr->lField[SUCCESS_VALUE]));

	if (dieRollPtr->lField[COMMON_BEEPTYPE1] >= g_iSoundList)
		dieRollPtr->lField[COMMON_BEEPTYPE1] = 0;
	if (dieRollPtr->lField[COMMON_BEEPTYPE2] >= g_iSoundList)
		dieRollPtr->lField[COMMON_BEEPTYPE2] = 0;

	InitDynamicPopup(pForm, IDC_BEEP_TYPE1, IDC_BEEP_LIST1, g_drCustom.lField[COMMON_BEEPTYPE1], g_pSoundList, g_iSoundList);
	InitDynamicPopup(pForm, IDC_BEEP_TYPE2, IDC_BEEP_LIST2, g_drCustom.lField[COMMON_BEEPTYPE2], g_pSoundList, g_iSoundList);
	InitDynamicPopup(pForm, IDC_COMPARE_TYPE1, IDC_COMPARE_LIST1, g_drCustom.lField[COMMON_BEEPCOMPARE1], g_pCompareList, g_iCompareList);
	InitDynamicPopup(pForm, IDC_COMPARE_TYPE2, IDC_COMPARE_LIST2, g_drCustom.lField[COMMON_BEEPCOMPARE2], g_pCompareList, g_iCompareList);
	InitDynamicPopup(pForm, IDC_SUCCESS_TYPE, IDC_SUCCESS_LIST, g_drCustom.lField[SUCCESS_COMPARE], g_pCompareList, g_iCompareList);
}
