#include "StdInc.h"
#include "Database.h"

DmOpenRef CreateDatabase (UInt32 ulAppId, UInt32 ulDatabaseId)
{
	DmOpenRef hDatabase;
	Char szDatabaseName [10];

	MemMove(szDatabaseName, &ulAppId, 4);
	*(szDatabaseName+4) = '-';
	MemMove((szDatabaseName+5), &ulDatabaseId, 4);
	*(szDatabaseName+9) = '\0';

	if (DmCreateDatabase(0, szDatabaseName, ulAppId, ulDatabaseId, false)) 
		return 0;

	return OpenDatabase(ulAppId, ulDatabaseId);
}

DmOpenRef OpenDatabase (UInt32 ulAppId, UInt32 ulDatabaseId)
{
	return DmOpenDatabaseByTypeCreator(ulDatabaseId, ulAppId, dmModeReadWrite);
}

Boolean CloseDatabase (DmOpenRef hDatabase)
{
	if (hDatabase)
	{
		// Close the database
//		DmResetRecordStates(hDatabase); // This crashes the app.
		DmCloseDatabase(hDatabase);
		return 1;
	}

	return 0;
}

Boolean DeleteDatabase (UInt32 ulAppId, UInt32 ulDatabaseId)
{
	Char szDatabaseName [10];
	LocalID vLocalId;

	MemMove(szDatabaseName, &ulAppId, 4);
	*(szDatabaseName+4) = '-';
	MemMove((szDatabaseName+5), &ulDatabaseId, 4);
	*(szDatabaseName+9) = '\0';

	vLocalId = DmFindDatabase(0, szDatabaseName);
	if (vLocalId == 0)
		return 0;
	if (DmDeleteDatabase(0, vLocalId))
		return 0;

	return 1;
}

Boolean SetAttributesDatabase (DmOpenRef hDatabase, UInt16 uAttributes, DbAttribType eDbAttribType)
{
	LocalID vLocalID;
	UInt16 uOldAttributes, uCardNumber;

	if (hDatabase == 0)
		return 0;

	if (DmOpenDatabaseInfo(hDatabase, &vLocalID, NULL, NULL, &uCardNumber, NULL) != 0)
		return 0;

	if (DmDatabaseInfo(uCardNumber, vLocalID, NULL, &uOldAttributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return 0;

	switch (eDbAttribType)
	{
	case DbAttribTypeRemove:
		uOldAttributes &= ~uAttributes;
		break;
	case DbAttribTypeOverwrite:
		uOldAttributes = uAttributes;
		break;
	default:  // default is DbAttribTypeAdd.
		uOldAttributes |= uAttributes;
	}

	if (DmSetDatabaseInfo(uCardNumber, vLocalID, NULL, &uOldAttributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return 0;

	return 1;
}

Boolean SetNameDatabase (DmOpenRef hDatabase, Char *pName)
{
	LocalID vLocalID;
	UInt16 uCardNumber;
	Char szName [dmDBNameLength];
	Int16 iNameSize;

	if ((hDatabase == 0) || (pName == 0))
		return 0;

	iNameSize = StrLen(pName);
	if (iNameSize == 0)
		return 0;

	// Truncate the name if needed.
	MemMove(szName, pName, ((iNameSize < dmDBNameLength) ? iNameSize : dmDBNameLength-1));
	szName[dmDBNameLength-1] = '\0';

	if (DmOpenDatabaseInfo(hDatabase, &vLocalID, NULL, NULL, &uCardNumber, NULL) != 0)
		return 0;

	if (DmSetDatabaseInfo(uCardNumber, vLocalID, szName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return 0;

	return 1;
}

Boolean ReadRecord (DmOpenRef hDatabase, UInt16 uiIndex, MemPtr pVoid, UInt32 ulLength)
{
	MemHandle hRecord;
	MemPtr pRecord;

	if (hDatabase)
	{
		if (DmNumRecords(hDatabase) > uiIndex)
		{
			hRecord = DmGetRecord(hDatabase, uiIndex);
			// If the record is smaller read in a partial record.
			if (ulLength > MemHandleSize(hRecord))
				ulLength = MemHandleSize(hRecord);
			pRecord = MemHandleLock(hRecord);
			MemMove(pVoid, pRecord, ulLength);
			MemHandleUnlock(hRecord); 
			DmReleaseRecord(hDatabase, uiIndex, false);  
			return 1;
		}
	}

	return 0;
}

Boolean OverwriteRecord (DmOpenRef hDatabase, UInt16 uiIndex, MemPtr pVoid, UInt32 ulLength)
{
	MemHandle hRecord;
	MemPtr pRecord;

	if (hDatabase)
	{
		// Does a record exist?
		if (DmNumRecords(hDatabase) > uiIndex)
		{
			hRecord = DmGetRecord(hDatabase, uiIndex);
			if (ulLength == MemHandleSize(hRecord))
			{
				// The same size record, overwrite it.
				pRecord = MemHandleLock(hRecord);
				DmWrite(pRecord, 0, pVoid, ulLength);
				MemHandleUnlock(hRecord); 
				DmReleaseRecord(hDatabase, uiIndex, false);
				return 1;
			}
			else
			{
				// New record erase it.
				DmReleaseRecord(hDatabase, uiIndex, false);
				DmRemoveRecord(hDatabase, uiIndex);
			}
		}
		// Write a new record.
		InsertRecord(hDatabase, uiIndex, pVoid, ulLength);
		return 1;
	}

	return 0;
}

Boolean InsertRecord (DmOpenRef hDatabase, UInt16 uiIndex, MemPtr pVoid, UInt32 ulLength)
{
	MemHandle hRecord;
	MemPtr pRecord;

	if (hDatabase)
	{
		// Write the new record.
		hRecord = DmNewRecord(hDatabase, &uiIndex, ulLength);
		pRecord = MemHandleLock(hRecord);
		DmWrite(pRecord, 0, pVoid, ulLength);
		MemHandleUnlock(hRecord); 
		DmReleaseRecord(hDatabase, uiIndex, true);  
		return 1;
	}

	return 0;
}

Boolean DeleteRecord (DmOpenRef hDatabase, UInt16 uiIndex)
{
	if (hDatabase) 
	{
		DmRemoveRecord(hDatabase, uiIndex);
	}

	return 1;
}

UInt16 GetRecordCount (DmOpenRef hDatabase)
{
	if (hDatabase) 
	{
		return DmNumRecords(hDatabase);
	}

	return 0;
}

