#include "StdInc.h"
#include "ListObj.h"

void ListConstruct (ListObject *pList)
{
	pList->pNext = pList;
	pList->pPrev = pList;
	pList->vData = 0;
}

void ListDestruct (ListObject *pList)
{
	ListClear(pList);
}

void ListAddFront (ListObject *pList, MemPtr pData, Int16 iSize)
{
	ListNodePtr pNewNode;
	Int16 iExtra;
	
	iExtra = (iSize > sizeof(pNewNode->vData)) ? iSize - sizeof(pNewNode->vData) : 0;
	pNewNode = (ListNodePtr)MemPtrNew(sizeof(ListNode) + iExtra);
	pNewNode->pNext = pList->pNext;
	pNewNode->pPrev = pList;

	if (pData)
		MemMove(&pNewNode->vData, pData, iSize);

	((ListNodePtr)pNewNode->pNext)->pPrev = pNewNode;
	((ListNodePtr)pNewNode->pPrev)->pNext = pNewNode;
	pList->vData++;
}

void ListAddRear (ListObject *pList, MemPtr pData, Int16 iSize)
{
	ListNodePtr pNewNode;
	Int16 iExtra;
	
	iExtra = (iSize > sizeof(pNewNode->vData)) ? iSize - sizeof(pNewNode->vData) : 0;
	pNewNode = (ListNodePtr)MemPtrNew(sizeof(ListNode) + iExtra);
	pNewNode->pNext = pList;
	pNewNode->pPrev = pList->pPrev;

	if (pData)
		MemMove(&pNewNode->vData, pData, iSize);

	((ListNodePtr)pNewNode->pNext)->pPrev = pNewNode;
	((ListNodePtr)pNewNode->pPrev)->pNext = pNewNode;
	pList->vData++;
}

void ListDelete (ListObject *pList, Int16 iIndex)
{
	ListNodePtr pCurrent;

	pCurrent = (ListNodePtr)pList->pNext;
	while (pCurrent != pList)
	{	
		if (iIndex-- == 0)
		{
			((ListNodePtr)pCurrent->pNext)->pPrev = (ListNodePtr)pCurrent->pPrev;
			((ListNodePtr)pCurrent->pPrev)->pNext = (ListNodePtr)pCurrent->pNext;
			MemPtrFree(pCurrent);
			pList->vData--;
			return;
		}
		pCurrent = (ListNodePtr)pCurrent->pNext;
	}
}

void ListClear (ListObject *pList)
{
	ListNodePtr pCurrent, pTemp;

	pCurrent = (ListNodePtr)pList->pNext;
	while (pCurrent != pList)
	{	
		pTemp = pCurrent;
		pCurrent = (ListNodePtr)pCurrent->pNext;
		MemPtrFree(pTemp);
	}

	pList->pNext = pList;
	pList->pPrev = pList;
	pList->vData = 0;
}

MemPtr ListGetData (ListObject *pList, Int16 iIndex)
{
	ListNodePtr pCurrent;

	pCurrent = (ListNodePtr)pList->pNext;
	while (pCurrent != pList)
	{
		if (iIndex-- == 0)
			return (MemPtr)&pCurrent->vData;
		pCurrent = (ListNodePtr)pCurrent->pNext;
	}

	return 0;
}

Int16 ListGetCount (ListObject *pList)
{
	return pList->vData;
}
