#include "StdInc.h"
#include "Misc.h"
#include "RollFunc.h"

#define MAX_LIST		5

typedef struct _tagRollList
{
	MemHandle hHandle;
	Int16 iTotalSize;
	Int16 iCurrentSize;
	Int16 iSorted;
} RollList;
typedef RollList *RollListPtr;

RollList g_vRollStack [MAX_LIST];
Int16 g_iStackIndex;

/***************************************************************\
 *                     Helper functions                        *
\***************************************************************/

Int16 SortListCompare (void *ptr1, void *ptr2, Int32 other)
{
    return (*((Int16*)ptr2) - *((Int16*)ptr1));
}

/***************************************************************\
 *                  Construction functions                     *
\***************************************************************/

void RollFuncConstructor ()
{
	Int16 i;

	for (i=0; i<MAX_LIST; i++)
	{
		g_vRollStack [g_iStackIndex].hHandle = 0;
		g_vRollStack [g_iStackIndex].iTotalSize = 0;
		g_vRollStack [g_iStackIndex].iCurrentSize = 0;
		g_vRollStack [g_iStackIndex].iSorted = 0;
	}
	g_iStackIndex = 0;
}

void RollFuncDestructor ()
{
	Int16 i;

	for (i=0; i<MAX_LIST; i++)
	{
		g_iStackIndex = i;
		DestroyList();
	}
	g_iStackIndex = 0;
}

/***************************************************************\
 *                      List functions                         *
\***************************************************************/

void CopyList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle == 0)
		return;

	if (g_iStackIndex < (MAX_LIST - 1))
	{
		Int16 *pSrc, *pDest;

		PushList();
		CreateList1(g_vRollStack [g_iStackIndex-1].iTotalSize);

		// Lock the lists.
		pSrc = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex-1].hHandle);
		pDest = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);

		// Copy the data.
		MemMove(pDest, pSrc, (g_vRollStack [g_iStackIndex-1].iTotalSize * sizeof(Int16)));

		// Copy the stats of the list.
		g_vRollStack [g_iStackIndex].iCurrentSize = g_vRollStack [g_iStackIndex-1].iCurrentSize;
		g_vRollStack [g_iStackIndex].iSorted = g_vRollStack [g_iStackIndex-1].iSorted;

		MemHandleUnlock(g_vRollStack [g_iStackIndex-1].hHandle);
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}
}

void PushList ()
{
	if (g_iStackIndex < (MAX_LIST - 1))
		g_iStackIndex++;
}

void PopList ()
{
	if (g_iStackIndex > 0)
		g_iStackIndex--;
}

void CreateList1 (Int16 iSize)
{
	CreateList3(iSize, 0, 0, 0);
}

void CreateList2 (Int16 iSize, Int16 iDie)
{
	CreateList3(iSize, 1, iDie, 0);
}

void CreateList3 (Int16 iSize, Int16 iNumber, Int16 iDie, Int16 iModifier)
{
	Int16 *pInt;

	// Limit checks
	if (iSize < 0)
		iSize = 1;
	if (iNumber < 0)
		iNumber = 0;
	if (iDie < 0)
		iDie = 0;

	// Kill the old list.
	DestroyList();

	// Get the list data and check for errors.
	g_vRollStack [g_iStackIndex].hHandle = MemHandleNew(iSize * sizeof(Int16));
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return;

	// Initialize the list
	pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	if (!iNumber || !iSize)
	{
		// Just initialize the list to zero
		MemSet(pInt, (iSize * sizeof(Int16)), '\0');
	}
	else
	{
		// Initialize the list to a set of die rolls.
		Int16 i;

		for (i=0; i<iSize; i++)
		{
			*pInt++ = CreateRoll(iNumber, iDie, iModifier);
		}
	}
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	g_vRollStack [g_iStackIndex].iTotalSize = iSize;
	g_vRollStack [g_iStackIndex].iCurrentSize = iSize;
	g_vRollStack [g_iStackIndex].iSorted = 0;
}

void DestroyList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle)
		MemHandleFree(g_vRollStack [g_iStackIndex].hHandle);
	g_vRollStack [g_iStackIndex].hHandle = 0;
	g_vRollStack [g_iStackIndex].iTotalSize = 0;
	g_vRollStack [g_iStackIndex].iCurrentSize = 0;
	g_vRollStack [g_iStackIndex].iSorted = 0;
}

Int16 GetTotalSizeList ()
{
	return g_vRollStack [g_iStackIndex].iTotalSize;
}

Int16 GetCurrentSizeList ()
{
	return g_vRollStack [g_iStackIndex].iCurrentSize;
}

Int16 GetElementList (Int16 iIndex)
{
	Int16 *pInt;
	Int16 lValue;

	if (!g_vRollStack [g_iStackIndex].hHandle || (iIndex >= g_vRollStack [g_iStackIndex].iCurrentSize))
		return 0;

	// Get the value of the element
	pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	lValue = pInt [iIndex];
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);

	return lValue;
}

void SetElementList (Int16 iIndex, Int16 lValue)
{
	Int16 *pInt;

	if (!g_vRollStack [g_iStackIndex].hHandle || (iIndex >= g_vRollStack [g_iStackIndex].iCurrentSize))
		return;

	// St the value of the element
	pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
	pInt [iIndex] = lValue;
	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
}

Int16 *GetList ()
{
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return 0;

	return (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
}

void ReleaseList ()
{
	if (!g_vRollStack [g_iStackIndex].hHandle)
		return;

	MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
}

Int32 SumList ()
{
	Int16 *pInt;
	Int32 lValue;
	Int16 i;

	lValue = 0;
	if (g_vRollStack [g_iStackIndex].hHandle)
	{
		pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		for (i=0; i<g_vRollStack [g_iStackIndex].iCurrentSize; i++)
		{
			lValue += pInt [i];
		}
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}

	return lValue;
}

Int32 MeanList ()
{
	Int32 lValue;

	if (g_vRollStack [g_iStackIndex].iCurrentSize == 0)
		return 0;

	// Mean with rounding
	lValue = SumList();
	lValue *= 10;
	lValue = (lValue / g_vRollStack [g_iStackIndex].iCurrentSize) + 5;
	lValue /= 10;

	return lValue;
}

Int32 MedianList ()
{
	Int32 lValue;

	if (g_vRollStack [g_iStackIndex].iCurrentSize == 0)
		return 0;

	SortList();

	// Median with rounding
	if (g_vRollStack [g_iStackIndex].iCurrentSize % 2)
	{
		lValue = GetElementList(g_vRollStack [g_iStackIndex].iCurrentSize / 2);
	}
	else
	{
		lValue = GetElementList(g_vRollStack [g_iStackIndex].iCurrentSize / 2);
		lValue += GetElementList((g_vRollStack [g_iStackIndex].iCurrentSize / 2) - 1);
		lValue *= 10;
		lValue = (lValue / 2) + 5;
		lValue /= 10;
	}

	return lValue;
}

Int32 ModeList ()
{
	Int32 lValue;

	lValue = 0;
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
	{
		Int16 iLargestRun, iCurrentRun, iNumberOfValues, i;
		Int16 lLastValue;
		Int16 *pInt;

		SortList();

		pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		if (g_vRollStack [g_iStackIndex].iCurrentSize == 1)
		{
			lValue = *pInt;
		}
		else
		{
			iLargestRun = 1;
			iCurrentRun = 1;
			iNumberOfValues = 0;
			lLastValue = *pInt;
			for (i=1; i<g_vRollStack [g_iStackIndex].iCurrentSize; i++)
			{
				if (pInt [i] != lLastValue)
				{
					if (iCurrentRun == iLargestRun)
					{
						lValue += lLastValue;
						iNumberOfValues++;
					}
					else if (iCurrentRun > iLargestRun)
					{
						lValue = lLastValue;
						iNumberOfValues = 1;
						iLargestRun = iCurrentRun;
					}
					iCurrentRun = 0;
				}
				lLastValue = pInt [i];
				iCurrentRun++;
			}
			if (iCurrentRun == iLargestRun)
			{
				lValue += lLastValue;
				iNumberOfValues++;
			}
			else if (iCurrentRun > iLargestRun)
			{
				lValue = lLastValue;
				iNumberOfValues = 1;
				iLargestRun = iCurrentRun;
			}

			lValue *= 10;
			lValue = (lValue / iNumberOfValues) + 5;
			lValue /= 10;
		}
		
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}

	return lValue;
}

void SizeList (Int16 iSize)
{
	if ((iSize < 0) || (iSize >= g_vRollStack [g_iStackIndex].iTotalSize))
		return;
	g_vRollStack [g_iStackIndex].iCurrentSize = iSize;
}

void SortList ()
{
	if (g_vRollStack [g_iStackIndex].iSorted == 0)
	{
		if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
		{
			UInt8 *pByte;

			pByte = (UInt8*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
			SysQSort(pByte, g_vRollStack [g_iStackIndex].iTotalSize, sizeof(Int16), SortListCompare, 0);
			MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);

			// It's now sorted, no need to resort.
			g_vRollStack [g_iStackIndex].iSorted = 1;
		}
	}
}

void ShuffleList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 0))
	{
		UInt8 *pByte;

		pByte = (UInt8*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
        QShuffle(pByte, g_vRollStack [g_iStackIndex].iCurrentSize, sizeof(Int16));
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);

		g_vRollStack [g_iStackIndex].iSorted = 0;
	}
}

void ReverseList ()
{
	if (g_vRollStack [g_iStackIndex].hHandle && (g_vRollStack [g_iStackIndex].iCurrentSize > 1))
	{
		Int16 *pInt;
		Int16 lValue;
		Int16 iStart, iEnd;

		iStart = 0;
		iEnd = g_vRollStack [g_iStackIndex].iCurrentSize - 1;

		pInt = (Int16*)MemHandleLock(g_vRollStack [g_iStackIndex].hHandle);
		while (iStart < iEnd)
		{
			lValue = pInt [iStart];
			pInt [iStart++] = pInt [iEnd];
			pInt [iEnd--] = lValue;
		}
		MemHandleUnlock(g_vRollStack [g_iStackIndex].hHandle);
	}
}

/***************************************************************\
 *                     Field functions                         *
\***************************************************************/

Int16 CreateRoll (Int16 iNumber, Int16 iDie, Int16 iModifier)
{
	Int16 lValue = 0;

	// Do a single die roll.
	while (iNumber-- > 0)
    {
        lValue += (Int16)(SysRandom(0) % iDie) + iModifier + 1;
    }

	return lValue;
}
