#include "StdInc.h"
#include "RollEm.h"

#include "RollSets.h"
#include "RollTypes.h"

#define TEXT_X			8
#define TEXT_Y			35
#define TEXT_WIDTH		136
#define TEXT_HEIGHT		91
#define TEXT_SPACING	2

// Structs 
typedef struct _tagRollResultsMemberVars
{
	Int16 g_iResultRowListPos;
	Int16 g_iResultRowListRows;
	Int16 g_iResultRowListCols;
	Int16 g_iResultRowCurrentSize;
} RollResultsMemberVars;
typedef RollResultsMemberVars *RollResultsMemberVarsPtr;

// Vars
RollResultsMemberVarsPtr g_pRollResultsMemberVars;

// Funcs
Boolean RollResultsProc (EventPtr pEvent);
void DrawRollResults ();

void InitResultRowList (FormPtr pForm);
void DrawResultRowList ();
void DrawResultRow (Char *pText, RectanglePtr pRect, Int32 lValue);
void MoveResultRowList (Int16 iDirection);
Boolean OpenResultSet (FormPtr pForm, EventPtr pEvent);

UInt16 DoModalRollResults ()
{
	FormPtr pForm, pOldForm;
	UInt16 wResult;

	wResult = 0;
	pOldForm = FrmGetActiveForm();
	if (pForm = FrmInitForm(IDF_RESULTS))
	{
		// Pre form stuff.
		FrmSetActiveForm(pForm);
		FrmDrawForm(pForm);

		// Do the every time initialization 
		//  of the Sound dialog here.
		g_pRollResultsMemberVars = MemPtrNew(sizeof(RollResultsMemberVars));
		if (g_pRollResultsMemberVars)
		{
			// Do the RollSet.
			InitPopup(pForm, IDC_RESULTS_NAME_TYPE, IDC_RESULTS_NAME_LIST, g_pPrefs->wRollDisplayName);
			InitPopup(pForm, IDC_RESULTS_STAT_TYPE, IDC_RESULTS_STAT_LIST, g_pPrefs->wRollDisplayStat);
			InitResultRowList(pForm);
			DrawRollResults();

			// Show the dialog.
			FrmSetEventHandler(pForm, (FormEventHandlerPtr)RollResultsProc);
			wResult = FrmDoDialog(pForm);

			MemPtrFree(g_pRollResultsMemberVars);
		}

		// We now return to our other window.
		if (pOldForm)
			FrmSetActiveForm(pOldForm);
		FrmDeleteForm(pForm);
	}

	return wResult;
}

Boolean RollResultsProc (EventPtr pEvent)
{
	FormPtr pForm = FrmGetActiveForm();
	Boolean bHandled = false;

	switch (pEvent->eType) 
    {
	case frmUpdateEvent:
	{
		FrmDrawForm(pForm);
		DrawRollResults();
		bHandled = true;
		break;
	}

	case popSelectEvent:
	{
		switch(pEvent->data.popSelect.listID)
		{
		case IDC_RESULTS_NAME_LIST:
			g_pPrefs->wRollDisplayName = pEvent->data.popSelect.selection;
			DrawResultRowList();
			break;
		case IDC_RESULTS_STAT_LIST:
			g_pPrefs->wRollDisplayStat = pEvent->data.popSelect.selection;
			MoveResultRowList(0);
			DrawResultRowList();
			break;
		}
		break;
	}

    case sclRepeatEvent:
	{
		MoveResultRowList(pEvent->data.sclRepeat.newValue - pEvent->data.sclRepeat.value);
		break;
	}
	
	case penDownEvent:  
	{
		bHandled = OpenResultSet(pForm, pEvent);
		break;
	}

	case keyDownEvent:
	{
        switch (pEvent->data.keyDown.chr)
        {
        case pageUpChr:
            MoveResultRowList(-1);
            break;
        case pageDownChr:
            MoveResultRowList(1);
            break;
        }
        break;
	}
	}

    return bHandled;
}

void InitResultRowList (FormPtr pForm)
{
	RecordInfo vRecordInfo;

	// Modify the lists to look better.
	SortList();
	ReverseList();

	g_pRollResultsMemberVars->g_iResultRowListPos = 0;
	g_pRollResultsMemberVars->g_iResultRowListRows = (TEXT_HEIGHT - (TEXT_SPACING*0)) / FntCharHeight();
	g_pRollResultsMemberVars->g_iResultRowListCols = 1;
	g_pRollResultsMemberVars->g_iResultRowCurrentSize = 0;

	if (OpenDatabaseRecordInfo(&vRecordInfo, ROLLEM_ID_APP, ROLLEM_DB_ROLL) == true)
	{
		g_pRollResultsMemberVars->g_iResultRowCurrentSize = DmNumRecords(vRecordInfo.hDatabase);

		// Close the database after wa are through with it.
		CloseDatabaseRecordInfo(&vRecordInfo);
	}
	MoveResultRowList(0);
}

void DrawRollResults ()
{
	DrawResultRowList();
}

void DrawResultRowList ()
{
	Int16 iCurrentX=TEXT_X + TEXT_SPACING, iCurrentY=TEXT_Y, i;
	Int16 iCharHeight, iEnd;
	RectangleType vRect;
	RecordInfo vRecordInfo;
	Char *pChar;
	Char szText [30];

	iCharHeight = FntCharHeight();

	vRect.topLeft.x = TEXT_X;
	vRect.topLeft.y = TEXT_Y;
	vRect.extent.x = TEXT_WIDTH;
	vRect.extent.y = TEXT_HEIGHT;

//	WinDrawRectangleFrame(rectangleFrame, &vRect);

	vRect.topLeft.x++;
	vRect.topLeft.y++;
	vRect.extent.x -= 5;
	vRect.extent.y -= 2;

	WinEraseRectangle(&vRect, 0);

	vRect.extent.y = iCharHeight;

	if (g_pPrefs->wRollDisplayStat == RollDisplayStatSubtotal)
	{
		Int32 lDice, lSubtotal;

		lDice = 0;
		lSubtotal = 0;
		if (OpenDatabaseRecordInfo(&vRecordInfo, ROLLEM_ID_APP, ROLLEM_DB_ROLL) == true)
		{
			// Use the index for saving rolls.
			vRecordInfo.vId = RECORDID_INDEX;
			vRecordInfo.vSubId = RECORDID_INDEX;

			for (i=0; i<g_pRollResultsMemberVars->g_iResultRowCurrentSize; i++)
			{
				vRecordInfo.vIndex = i;
				ReadRecordInfo(&vRecordInfo);

				pChar = ((Char*)vRecordInfo.vDatabase.pRecord) + sizeof(DieRoll);
				lDice += ((DieRollStatPtr)pChar)->lSize;
				lSubtotal += ((DieRollStatPtr)pChar)->lTotal;

				CloseRecordInfo(&vRecordInfo);
			}
			
			// Close the database after wa are through with it.
			CloseDatabaseRecordInfo(&vRecordInfo);

			DrawResultRow("Rolls", &vRect, g_pRollResultsMemberVars->g_iResultRowCurrentSize);
			vRect.topLeft.y += iCharHeight;
			DrawResultRow("Dice", &vRect, lDice);
			vRect.topLeft.y += iCharHeight;
			DrawResultRow("Total", &vRect, lSubtotal);
		}
	}
	else
	{
		Int32 lValue;

		if (OpenDatabaseRecordInfo(&vRecordInfo, ROLLEM_ID_APP, ROLLEM_DB_ROLL) == true)
		{
			// Use the index for saving rolls.
			vRecordInfo.vId = RECORDID_INDEX;
			vRecordInfo.vSubId = RECORDID_INDEX;

			// Figure out the ending point.
			iEnd = g_pRollResultsMemberVars->g_iResultRowListPos + g_pRollResultsMemberVars->g_iResultRowListRows;
			if (iEnd > g_pRollResultsMemberVars->g_iResultRowCurrentSize)
				iEnd = g_pRollResultsMemberVars->g_iResultRowCurrentSize;

			for (i=g_pRollResultsMemberVars->g_iResultRowListPos; i<iEnd; i++)
			{
				vRecordInfo.vIndex = i;
				ReadRecordInfo(&vRecordInfo);
				pChar = (Char*)vRecordInfo.vDatabase.pRecord;

				// Draw the description.
				switch(g_pPrefs->wRollDisplayName)
				{
				case RollDisplayNameTitle:
					StrCopy(szText, ((DieRollPtr)pChar)->szTitle);
					break;
				case RollDisplayNameRoll:
					CreateTitle(((DieRollPtr)pChar), szText, 20);
					break;
				}

				// Move to the stat.
				pChar += sizeof(DieRoll);

				// Draw the values that are selected.
				switch(g_pPrefs->wRollDisplayStat)
				{
				case RollDisplayStatTotal:
					lValue = ((DieRollStatPtr)pChar)->lTotal;
					break;
				case RollDisplayStatSum:
					lValue = ((DieRollStatPtr)pChar)->lSum;
					break;
				case RollDisplayStatMean:
					lValue = ((DieRollStatPtr)pChar)->lMean;
					break;
				case RollDisplayStatMedian:
					lValue = ((DieRollStatPtr)pChar)->lMedian;
					break;
				case RollDisplayStatMode:
					lValue = ((DieRollStatPtr)pChar)->lMode;
					break;
				}
				DrawResultRow(szText, &vRect, lValue);
				CloseRecordInfo(&vRecordInfo);

				// Move to the next position.
				vRect.topLeft.y += iCharHeight;
			}
			
			// Close the database after wa are through with it.
			CloseDatabaseRecordInfo(&vRecordInfo);
		}
	}
}

void DrawResultRow (Char *pText, RectanglePtr pRect, Int32 lValue)
{
	Char szText [30];

	// Draw the row title.
	DrawCharacters(pText, StrLen(pText), pRect, DC_LEFT, true);

	// Draw the row value.
	StrLToA(szText, lValue);
	DrawCharacters(szText, StrLen(szText), pRect, DC_RIGHT, true);
}

void MoveResultRowList (Int16 iDirection)
{
	FormPtr pForm = FrmGetActiveForm();
	ScrollBarPtr pScroll;
	Int16 iNewPos, iScrollSize;

	if (iDirection != 0)
	{
		iNewPos = g_pRollResultsMemberVars->g_iResultRowListPos + iDirection;

		if ((iNewPos + g_pRollResultsMemberVars->g_iResultRowListRows) > g_pRollResultsMemberVars->g_iResultRowCurrentSize)
			iNewPos = g_pRollResultsMemberVars->g_iResultRowCurrentSize - g_pRollResultsMemberVars->g_iResultRowListRows;
		if (iNewPos < 0)
			iNewPos = 0;

		if (iNewPos != g_pRollResultsMemberVars->g_iResultRowListPos)
		{
			g_pRollResultsMemberVars->g_iResultRowListPos = iNewPos;
			DrawResultRowList();
		}
	}

	if (g_pPrefs->wRollDisplayStat == RollDisplayStatSubtotal)
	{
		iScrollSize = 0;
	}
	else
	{
		iScrollSize = g_pRollResultsMemberVars->g_iResultRowCurrentSize - g_pRollResultsMemberVars->g_iResultRowListRows;
		if (iScrollSize < 0)
			iScrollSize = 0;
	}

	pScroll = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_RESULTS_SCROLL));
	SclSetScrollBar (pScroll, g_pRollResultsMemberVars->g_iResultRowListPos, 0, iScrollSize, g_pRollResultsMemberVars->g_iResultRowListRows);
}

Boolean OpenResultSet (FormPtr pForm, EventPtr pEvent)
{
	RectangleType vRect;

	if (g_pRollResultsMemberVars->g_iResultRowCurrentSize == 0)
		return false;
	if (g_pPrefs->wRollDisplayStat == RollDisplayStatSubtotal)
		return false;

	vRect.topLeft.x = TEXT_X;
	vRect.topLeft.y = TEXT_Y;
	vRect.extent.x = TEXT_WIDTH;
	vRect.extent.y = g_pRollResultsMemberVars->g_iResultRowListRows;
	if (g_pRollResultsMemberVars->g_iResultRowCurrentSize < vRect.extent.y) 
		vRect.extent.y = g_pRollResultsMemberVars->g_iResultRowCurrentSize;
	vRect.extent.y *= FntLineHeight();

	if (!RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vRect))
		return false;

	DoModalRollSet(((pEvent->screenY - TEXT_Y) / FntLineHeight()) + g_pRollResultsMemberVars->g_iResultRowListPos);

	return true;
}
