#include "StdInc.h"
#include "RollEm.h"
#include "AdvCustom.h"
#include "RollFunc.h"
#include "RollTypes.h"

Int32 RollDiceNormal (FormType *pForm, DieRoll *pDieRoll);
Int32 RollDiceSuccess (FormType *pForm, DieRoll *pDieRoll);
Int32 RollDiceOpenEnded (FormType *pForm, DieRoll *pDieRoll);
Int32 RollDiceThaco (FormType *pForm, DieRoll *pDieRoll);
Int32 RollDiceAdditive (FormType *pForm, DieRoll *pDieRoll);
Int32 RollDiceSubtractive (FormType *pForm, DieRoll *pDieRoll);

void CreateTitleNormal (DieRoll *pDieRoll, Char *pTitle);
void CreateTitleSuccess (DieRoll *pDieRoll, Char *pTitle);
void CreateTitleOpenEnded (DieRoll *pDieRoll, Char *pTitle);
void CreateTitleThaco (DieRoll *pDieRoll, Char *pTitle);
void CreateTitleAdditive (DieRoll *pDieRoll, Char *pTitle);
void CreateTitleSubtractive (DieRoll *pDieRoll, Char *pTitle);

void RollDice (FormType *pForm, DieRoll *pDieRoll, Results *pResults)
{
	switch(pDieRoll->lField[COMMON_ROLLTYPE])
	{
	case RollTypeNormal:
		pResults->vLastRollStat.lTotal = RollDiceNormal(pForm, pDieRoll);
		break;
	case RollTypeSuccess:
		pResults->vLastRollStat.lTotal = RollDiceSuccess(pForm, pDieRoll);
		break;
	case RollTypeOpenEnded:
		pResults->vLastRollStat.lTotal = RollDiceOpenEnded(pForm, pDieRoll);
		break;
	case RollTypeThaco:
		pResults->vLastRollStat.lTotal = RollDiceThaco(pForm, pDieRoll);
		break;
	case RollTypeAdditive:
		pResults->vLastRollStat.lTotal = RollDiceAdditive(pForm, pDieRoll);
		break;
	case RollTypeSubtractive:
		pResults->vLastRollStat.lTotal = RollDiceSubtractive(pForm, pDieRoll);
		break;
	}
	pResults->vLastRollStat.lSize = GetCurrentSizeList();
	pResults->vLastRollStat.lTotalSize = GetCurrentSizeList();
	pResults->vLastRollStat.lSum = SumList();

	// Make a copy of the lists so we don't destroy the real one.
	CopyList();

	// Do our statistic stuff.
	pResults->vLastRollStat.lMean = MeanList();
	pResults->vLastRollStat.lMedian = MedianList();
	pResults->vLastRollStat.lMode = ModeList();

	// We don't need the copy anymore.
	PopList();
}

void CreateTitle (DieRoll *pDieRoll, Char *pTitle, UInt16 wLen)
{
	Char szTemp[MAX_TEMPSTRING_LEN];

	switch(pDieRoll->lField[COMMON_ROLLTYPE])
	{
	case RollTypeNormal:
		CreateTitleNormal(pDieRoll, szTemp);
		break;
	case RollTypeSuccess:
		CreateTitleSuccess(pDieRoll, szTemp);
		break;
	case RollTypeOpenEnded:
		CreateTitleOpenEnded(pDieRoll, szTemp);
		break;
	case RollTypeThaco:
		CreateTitleThaco(pDieRoll, szTemp);
		break;
	case RollTypeAdditive:
		CreateTitleAdditive(pDieRoll, szTemp);
		break;
	case RollTypeSubtractive:
		CreateTitleSubtractive(pDieRoll, szTemp);
		break;
	}

	if (wLen > MAX_TEMPSTRING_LEN)
		wLen = MAX_TEMPSTRING_LEN;
	StrNCopy(pTitle, szTemp, --wLen);
	pTitle[wLen] = '\0';
}

Int32 RollDiceNormal (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult = pDieRoll->lField[NORMAL_MODIFIER];

	CreateList2(pDieRoll->lField[NORMAL_NUMBER], pDieRoll->lField[NORMAL_DIE]);
    if ((pDieRoll->lField[NORMAL_BEST] > 0) && (pDieRoll->lField[NORMAL_BEST] < pDieRoll->lField[NORMAL_NUMBER]))
	{
		SortList();
		SizeList(pDieRoll->lField[NORMAL_BEST]);
	}
	iResult += SumList();

    return iResult;
}

Int32 RollDiceSuccess (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult = pDieRoll->lField[SUCCESS_MODIFIER];
	Int16 iSize, i; 

	CreateList2(pDieRoll->lField[SUCCESS_NUMBER], pDieRoll->lField[SUCCESS_DIE]);
    if ((pDieRoll->lField[SUCCESS_BEST] > 0) && (pDieRoll->lField[SUCCESS_BEST] < pDieRoll->lField[SUCCESS_NUMBER]))
	{
		SortList();
		SizeList(pDieRoll->lField[SUCCESS_BEST]);
	}

	iSize = GetCurrentSizeList();
	for (i=0; i<iSize; i++)
	{
		if (CheckValue(pDieRoll->lField[SUCCESS_COMPARE], pDieRoll->lField[SUCCESS_VALUE], GetElementList(i)))
			iResult++;
	}

    return iResult;
}

Int32 RollDiceOpenEnded (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult, iHighCompareValue, iLowCompareValue;
	Int16 iCount, *pList;
	UInt8 uHighCompareType, uLowCompareType;

	CreateList1(pDieRoll->lField[OPENEND_MAXROLLS]);
	pList = GetList();

	uHighCompareType = pDieRoll->lField[OPENEND_HIGHCOMPARE];
	iHighCompareValue = pDieRoll->lField[OPENEND_HIGHVALUE];
	uLowCompareType = pDieRoll->lField[OPENEND_LOWCOMPARE];
	iLowCompareValue = pDieRoll->lField[OPENEND_LOWVALUE];

	iCount = 0;
	pList[iCount] = CreateRoll(1, pDieRoll->lField[OPENEND_DIE], 0);
	iResult = pList[iCount++] + pDieRoll->lField[OPENEND_MODIFIER];
	if (CheckValue(uHighCompareType, iHighCompareValue, pList[iCount-1]) == true)
	{
		do 
		{
			pList[iCount] = CreateRoll(1, pDieRoll->lField[OPENEND_DIE], 0);
			iResult += pList[iCount++];
		} while ((iCount < pDieRoll->lField[OPENEND_MAXROLLS]) && (CheckValue(uHighCompareType, iHighCompareValue, pList[iCount-1]) == true));
	}
	else if (CheckValue(uLowCompareType, iLowCompareValue, pList[iCount-1]) == true)
	{
		do 
		{
			pList[iCount] = CreateRoll(1, pDieRoll->lField[OPENEND_DIE], 0);
			iResult -= pList[iCount++];
		} while ((iCount < pDieRoll->lField[OPENEND_MAXROLLS]) && (CheckValue(uHighCompareType, iHighCompareValue, pList[iCount-1]) == true));
	}
	ReleaseList();
	SizeList(iCount);

    return iResult;
}

Int32 RollDiceThaco (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult = pDieRoll->lField[THACO_MODIFIER];

	CreateList2(pDieRoll->lField[THACO_NUMBER], pDieRoll->lField[THACO_DIE]);
	iResult += SumList();
	iResult = pDieRoll->lField[THACO_VALUE] - iResult;

    return iResult;
}

Int32 RollDiceAdditive (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult = pDieRoll->lField[ADDITIVE_MODIFIER];
	Int16 i, iPos, *pList;

	CreateList1(pDieRoll->lField[ADDITIVE_NUMBER1] + pDieRoll->lField[ADDITIVE_NUMBER2]);
	pList = GetList();
	iPos = 0;
	for (i=0; i<pDieRoll->lField[ADDITIVE_NUMBER1]; i++)
	{
		pList[iPos] = CreateRoll(1, pDieRoll->lField[ADDITIVE_DIE1], 0);
		iResult += pList[iPos++];
	}
	for (i=0; i<pDieRoll->lField[ADDITIVE_NUMBER2]; i++)
	{
		pList[iPos] = CreateRoll(1, pDieRoll->lField[ADDITIVE_DIE2], 0);
		iResult += pList[iPos++];
	}
	ReleaseList();

    return iResult;
}

Int32 RollDiceSubtractive (FormType *pForm, DieRoll *pDieRoll)
{
    Int32 iResult = pDieRoll->lField[ADDITIVE_MODIFIER];
	Int16 i, iPos, *pList;

	CreateList1(pDieRoll->lField[ADDITIVE_NUMBER1] + pDieRoll->lField[ADDITIVE_NUMBER2]);
	pList = GetList();
	iPos = 0;
	for (i=0; i<pDieRoll->lField[ADDITIVE_NUMBER1]; i++)
	{
		pList[iPos] = CreateRoll(1, pDieRoll->lField[ADDITIVE_DIE1], 0);
		iResult += pList[iPos++];
	}
	for (i=0; i<pDieRoll->lField[ADDITIVE_NUMBER2]; i++)
	{
		pList[iPos] = CreateRoll(1, pDieRoll->lField[ADDITIVE_DIE2], 0);
		iResult -= pList[iPos++];
	}
	ReleaseList();

    return iResult;
}

void CreateTitleNormal (DieRoll *pDieRoll, Char *pTitle)
{
	*pTitle = '\0';
    if (pDieRoll->lField[NORMAL_MODIFIER] != 0)
    {
		StrCopy(&pTitle[StrLen(pTitle)], "(");
	}

    if ((pDieRoll->lField[NORMAL_BEST] > 0) && (pDieRoll->lField[NORMAL_BEST] < pDieRoll->lField[NORMAL_NUMBER]))
    {
		// Add the best of section.
		StrPrintF(&pTitle[StrLen(pTitle)], "%ld of ", 
			pDieRoll->lField[NORMAL_BEST]);
    }

	// Add the 4d6 section.
	StrPrintF(&pTitle[StrLen(pTitle)], "%ldd%ld", 
		pDieRoll->lField[NORMAL_NUMBER],
		pDieRoll->lField[NORMAL_DIE]);

    if (pDieRoll->lField[NORMAL_MODIFIER] != 0)
    {
		// Add the modifier section..
		StrPrintF(&pTitle[StrLen(pTitle)], ")%s%ld", 
			(pDieRoll->lField[NORMAL_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[NORMAL_MODIFIER]);
    }
}

void CreateTitleSuccess (DieRoll *pDieRoll, Char *pTitle)
{
	*pTitle = '\0';
    if (pDieRoll->lField[SUCCESS_MODIFIER] != 0)
    {
		StrCopy(&pTitle[StrLen(pTitle)], "(");
	}

    if ((pDieRoll->lField[SUCCESS_BEST] > 0) && (pDieRoll->lField[SUCCESS_BEST] < pDieRoll->lField[SUCCESS_NUMBER]))
    {
		// Add the best of section.
		StrPrintF(&pTitle[StrLen(pTitle)], "%ld of ", 
			pDieRoll->lField[SUCCESS_BEST]);
    }

	// Add the 4d6 section.
	StrPrintF(&pTitle[StrLen(pTitle)], "%ldd%ld", 
		pDieRoll->lField[SUCCESS_NUMBER],
		pDieRoll->lField[SUCCESS_DIE]);

	if (pDieRoll->lField[SUCCESS_VALUE] > 0)
	{
		Char szComp [5];

		CompareString(pDieRoll->lField[SUCCESS_COMPARE], szComp);
		StrPrintF(&pTitle[StrLen(pTitle)], " %s %ld", 
			szComp,
			pDieRoll->lField[SUCCESS_VALUE]);
	}

    if (pDieRoll->lField[SUCCESS_MODIFIER] != 0)
    {
		// Add the modifier section..
		StrPrintF(&pTitle[StrLen(pTitle)], ")%s%ld", 
			(pDieRoll->lField[SUCCESS_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[SUCCESS_MODIFIER]);
    }
}

void CreateTitleOpenEnded (DieRoll *pDieRoll, Char *pTitle)
{
	Char szComp [5];

	*pTitle = '\0';
    if (pDieRoll->lField[OPENEND_MODIFIER] != 0)
    {
		StrCopy(&pTitle[StrLen(pTitle)], "(");
	}

	StrPrintF(&pTitle[StrLen(pTitle)], "d%ld", 
		pDieRoll->lField[OPENEND_DIE]);

    if (pDieRoll->lField[OPENEND_HIGHVALUE] != 0)
    {
		CompareString(pDieRoll->lField[OPENEND_HIGHCOMPARE], szComp);
		StrPrintF(&pTitle[StrLen(pTitle)], " Exp [%s%ld]", 
			szComp,
			pDieRoll->lField[OPENEND_HIGHVALUE]);
	}
    if (pDieRoll->lField[OPENEND_LOWVALUE] != 0)
    {
		CompareString(pDieRoll->lField[OPENEND_LOWCOMPARE], szComp);
		StrPrintF(&pTitle[StrLen(pTitle)], " Imp [%s%ld]", 
			szComp,
			pDieRoll->lField[OPENEND_LOWVALUE]);
	}

    if (pDieRoll->lField[OPENEND_MODIFIER] != 0)
    {
		// Add the modifier section..
		StrPrintF(&pTitle[StrLen(pTitle)], ")%s%ld", 
			(pDieRoll->lField[OPENEND_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[OPENEND_MODIFIER]);
    }
}

void CreateTitleThaco (DieRoll *pDieRoll, Char *pTitle)
{
	StrPrintF(pTitle, "%ldd%ld", 
		pDieRoll->lField[THACO_NUMBER],
		pDieRoll->lField[THACO_DIE]);

    if (pDieRoll->lField[THACO_MODIFIER] != 0)
    {
		// Add the modifier section..
		StrPrintF(&pTitle[StrLen(pTitle)], "%s%ld", 
			(pDieRoll->lField[THACO_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[THACO_MODIFIER]);
    }

	StrPrintF(&pTitle[StrLen(pTitle)], " Thac0 %ld", 
		pDieRoll->lField[THACO_VALUE]);
}

void CreateTitleAdditive (DieRoll *pDieRoll, Char *pTitle)
{
	StrPrintF(pTitle, "(%ldd%ld + %ldd%ld)", 
		pDieRoll->lField[ADDITIVE_NUMBER1],
		pDieRoll->lField[ADDITIVE_DIE1],
		pDieRoll->lField[ADDITIVE_NUMBER2],
		pDieRoll->lField[ADDITIVE_DIE2]);

    if (pDieRoll->lField[ADDITIVE_MODIFIER] != 0)
    {
		StrPrintF(&pTitle[StrLen(pTitle)], "%s%ld", 
			(pDieRoll->lField[ADDITIVE_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[ADDITIVE_MODIFIER]);
	}
}

void CreateTitleSubtractive (DieRoll *pDieRoll, Char *pTitle)
{
	StrPrintF(pTitle, "(%ldd%ld - %ldd%ld)", 
		pDieRoll->lField[ADDITIVE_NUMBER1],
		pDieRoll->lField[ADDITIVE_DIE1],
		pDieRoll->lField[ADDITIVE_NUMBER2],
		pDieRoll->lField[ADDITIVE_DIE2]);

    if (pDieRoll->lField[ADDITIVE_MODIFIER] != 0)
    {
		StrPrintF(&pTitle[StrLen(pTitle)], "%s%ld", 
			(pDieRoll->lField[ADDITIVE_MODIFIER] < 0) ? "" : "+",
			pDieRoll->lField[ADDITIVE_MODIFIER]);
	}
}
