#include "StdInc.h"
#include "SpinCtrl.h"

// Structs 
// Defines
#define FRAME_TYPE	0x0101

#define SPIN_LEFT	1
#define SPIN_RIGHT	-1

Int16 SpinCtrlRectangle (Int16 iX, Int16 iY, RectanglePtr pCtrlRect, RectanglePtr pRect);

void SpinCtrlConstructor (FormPtr pForm, UInt16 wSpinId)
{
}

void SpinCtrlDestructor (FormPtr pForm, UInt16 wSpinId)
{
}

void SpinCtrlSetUsable (FormPtr pForm, UInt16 wSpinId, Boolean bUsable)
{
	GadgetPtr pGadget;

	pGadget = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wSpinId));
	pGadget->attr.usable = bUsable;
	SpinCtrlDraw(pForm, wSpinId);
}

Boolean SpinCtrlGetUsable (FormPtr pForm, UInt16 wSpinId)
{
	GadgetPtr pGadget;

	pGadget = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wSpinId));
	return pGadget->attr.usable;
}

void SpinCtrlDrawCtrl (FormPtr pForm, Int16 iDir, RectangleType *pCtrlRect, Boolean bSelected)
{
	Int16 iX1, iX2, iY1, iY2, iWidth, i;

	if (IsRomVersion(VERSION_350))
	{
		// Draw the inside.
		WinSetForeColor(UIColorGetTableEntryIndex(((bSelected == true) ? UIObjectSelectedFill : UIObjectFill)));
		WinDrawRectangle(pCtrlRect, 0);

		// Draw the arrows.
		WinSetForeColor(UIColorGetTableEntryIndex(((bSelected == true) ? UIObjectSelectedForeground : UIObjectForeground)));
	}

	switch (iDir)
	{
	case SPIN_LEFT:
		iWidth = (pCtrlRect->extent.y < 5) ? 2 : 3;
		iX1 = pCtrlRect->topLeft.x + (pCtrlRect->extent.x / 2);
		iX2 = iX1;
		iY1 = pCtrlRect->topLeft.y + 1;
		for (i=0; i<iWidth; i++)
		{
			WinDrawLine(iX1, iY1, iX2, iY1);
			iX1--;
			iX2++;
			iY1++;
		}
		break;
	case SPIN_RIGHT:
		iWidth = (pCtrlRect->extent.y < 5) ? 2 : 3;
		iX1 = pCtrlRect->topLeft.x + (pCtrlRect->extent.x / 2);
		iX2 = iX1;
		iY2 = pCtrlRect->topLeft.y + pCtrlRect->extent.y - 2;
		for (i=0; i<iWidth; i++)
		{
			WinDrawLine(iX1, iY2, iX2, iY2);
			iX1--;
			iX2++;
			iY2--;
		}
		break;
	}
}

void SpinCtrlDraw(FormPtr pForm, UInt16 wSpinId)
{
	Boolean bVersion350 = IsRomVersion(VERSION_350);
	UInt16 wGadgetIndex = FrmGetObjectIndex(pForm, wSpinId);
	RectangleType vCtrlRect, vTempRect;
	GadgetPtr pGadget;

	FrmGetObjectBounds(pForm, wGadgetIndex, &vCtrlRect);

	pGadget = FrmGetObjectPtr(pForm, wGadgetIndex);
	if (pGadget->attr.usable)
	{
		if (bVersion350 == true)
		{
			// Save the draw state.
			WinPushDrawState();
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectFrame));
		}

		// Draw the outside.
		WinDrawRectangleFrame(FRAME_TYPE, &vCtrlRect);

		if ((vCtrlRect.extent.y % 2) == 1)
		{
			WinDrawLine(vCtrlRect.topLeft.x, 
				(vCtrlRect.topLeft.y + (vCtrlRect.extent.y / 2)),
				(vCtrlRect.topLeft.x + vCtrlRect.extent.x),
				(vCtrlRect.topLeft.y + (vCtrlRect.extent.y / 2)));
		}

		SpinCtrlRectangle(vCtrlRect.topLeft.x, vCtrlRect.topLeft.y, &vCtrlRect, &vTempRect);
		SpinCtrlDrawCtrl(pForm, SPIN_LEFT, &vTempRect, false);
		SpinCtrlRectangle(vCtrlRect.topLeft.x + vCtrlRect.extent.x - 1, vCtrlRect.topLeft.y + vCtrlRect.extent.y - 1, &vCtrlRect, &vTempRect);
		SpinCtrlDrawCtrl(pForm, SPIN_RIGHT, &vTempRect, false);

		if (bVersion350 == true)
		{
			// Restore the draw state.
			WinPopDrawState();
		}
	}
	else
	{
		InflateRect(&vCtrlRect, 1, 1);
		WinEraseRectangle(&vCtrlRect, 0);
	}
}

Int16 SpinCtrlRectangle (Int16 iX, Int16 iY, RectanglePtr pCtrlRect, RectanglePtr pRect)
{
	pRect->topLeft.x = pCtrlRect->topLeft.x;
	pRect->topLeft.y = pCtrlRect->topLeft.y;
	pRect->extent.x = pCtrlRect->extent.x;
	pRect->extent.y = (pCtrlRect->extent.y / 2);
	if (RctPtInRectangle(iX, iY, pRect))
		return SPIN_LEFT;

	pRect->topLeft.y = pCtrlRect->topLeft.y + pRect->extent.y + (pCtrlRect->extent.y % 2);
	pRect->extent.y = pCtrlRect->extent.y - pRect->extent.y - (pCtrlRect->extent.y % 2);
	if (RctPtInRectangle(iX, iY, pRect))
		return SPIN_RIGHT;

	pRect->extent.x = 0;
	pRect->extent.y = 0;

	return SPINCTRL_NOTAP;
}

Int16 SpinCtrlTap(FormPtr pForm, UInt16 wSpinId, EventPtr pEvent)
{
	Boolean bVersion350 = IsRomVersion(VERSION_350);
	UInt16 wGadgetIndex = FrmGetObjectIndex(pForm, wSpinId);
	GadgetPtr pGadget;
	RectangleType vCtrlRect, vLastRect, vTempRect;
	Int16 iResult, iLastResult;
	Int16 x, y;
	Boolean bPenDown;

	pGadget = FrmGetObjectPtr(pForm, wGadgetIndex);
	if (!pGadget->attr.usable)
		return SPINCTRL_NOTAP;

	FrmGetObjectBounds(pForm, wGadgetIndex, &vCtrlRect);
	if (!RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vCtrlRect))
		return SPINCTRL_NOTAP;

	iLastResult = SPINCTRL_NOTAP;
	vLastRect.extent.x = 0;
	vLastRect.extent.y = 0;

	do 
	{
		PenGetPoint(&x, &y, &bPenDown);
		iResult = SpinCtrlRectangle (/*pSpinData, */x, y, &vCtrlRect, &vTempRect);
		if (iLastResult != iResult)
		{
			if (bVersion350 == true)
			{
				WinPushDrawState();
				SpinCtrlDrawCtrl(pForm, iLastResult, &vLastRect, false);
				SpinCtrlDrawCtrl(pForm, iResult, &vTempRect, true);
				WinPopDrawState();
			}
			else
			{
				WinInvertRectangle(&vLastRect, 0);
				WinInvertRectangle(&vTempRect, 0);
			}

			vLastRect.topLeft.x = vTempRect.topLeft.x ;
			vLastRect.topLeft.y = vTempRect.topLeft.y;
			vLastRect.extent.x = vTempRect.extent.x;
			vLastRect.extent.y = vTempRect.extent.y;
			iLastResult = iResult;
		}
	} while (bPenDown);
	
	if (iResult != SPINCTRL_NOTAP)
	{
		SndPlaySystemSound(sndClick);
		if (bVersion350 == true)
		{
			WinPushDrawState();
			SpinCtrlDrawCtrl(pForm, iResult, &vLastRect, false);
			WinPopDrawState();
		}
		else
		{
			WinInvertRectangle(&vLastRect, 0);
		}
	}

	return iLastResult;
}
