#include "StdInc.h"
#include "StackObj.h"

void StackConstruct (StackObject *pStack)
{
	pStack->pNext = 0;
	pStack->vData = 0;
}

void StackDestruct (StackObject *pStack)
{
	StackClear(pStack);
}

void StackAddFront (StackObject *pStack, MemPtr pData, Int16 iSize)
{
	StackNodePtr pNewNode;
	Int16 iExtra;
	
	iExtra = (iSize > sizeof(pNewNode->vData)) ? iSize - sizeof(pNewNode->vData) : 0;
	pNewNode = (StackNodePtr)MemPtrNew(sizeof(StackNode) + iExtra);
	pNewNode->pNext = pStack->pNext;

	if (pData)
		MemMove(&pNewNode->vData, pData, iSize);

	pStack->pNext = pNewNode;
	pStack->vData++;
}

void StackAddRear (StackObject *pStack, MemPtr pData, Int16 iSize)
{
	StackNodePtr pCurrent, pNewNode;
	Int16 iExtra;
	
	iExtra = (iSize > sizeof(pNewNode->vData)) ? iSize - sizeof(pNewNode->vData) : 0;
	pNewNode = (StackNodePtr)MemPtrNew(sizeof(StackNode) + iExtra);

	if (pData)
		MemMove(&pNewNode->vData, pData, iSize);

	pCurrent = pStack;
	while (pCurrent->pNext)
		pCurrent = (StackNodePtr)pCurrent->pNext;

	pNewNode->pNext = pCurrent->pNext;
	pCurrent->pNext = pNewNode;
	pStack->vData++;
}

void StackDelete (StackObject *pStack, Int16 iIndex)
{
	StackNodePtr pCurrent, pPrev;

	pCurrent = pStack;
	while (pCurrent)
	{	
		pPrev = pCurrent;
		pCurrent = (StackNodePtr)pPrev->pNext;
		if (iIndex-- == 0)
		{
			pPrev->pNext = (StackNodePtr)pCurrent->pNext;
			MemPtrFree(pCurrent);
			pStack->vData--;
			return;
		}
	}
}

void StackClear (StackObject *pStack)
{
	StackNodePtr pCurrent, pTemp;

	pCurrent = (StackNodePtr)pStack->pNext;
	while (pCurrent)
	{
		pTemp = pCurrent;
		pCurrent = (StackNodePtr)pCurrent->pNext;
		MemPtrFree(pTemp);
	}

	pStack->pNext = 0;
	pStack->vData = 0;
}

MemPtr StackGetData (StackObject *pStack, Int16 iIndex)
{
	StackNodePtr pCurrent;

	pCurrent = (StackNodePtr)pStack->pNext;
	while (pCurrent)
	{
		if (iIndex-- == 0)
			return (MemPtr)&pCurrent->vData;
		pCurrent = (StackNodePtr)pCurrent->pNext;
	}

	return 0;
}

Int16 StackGetCount (StackObject *pStack)
{
	return pStack->vData;
}
