#include "StdInc.h"
#include "StrMath.h"

Int16 DecimalCount (Char *pStr)
{
	Char *pPtr;
	Int16 iCount = 0;
	Boolean bDecimal = 0;

	pPtr = pStr;
	while (*pPtr)
	{
		if (bDecimal)
			iCount++;
		if (*pPtr == '.')
			bDecimal = 1;
		pPtr++;
	}

	return iCount;
}

Char *MakeDecimal (Char *pStr, Int16 iPlaces, Boolean bRound)
{
	Char *pPtr;
	Int16 iCount = 0;
	Boolean bDecimal = 0;

	pPtr = pStr;
	while (*pPtr)
	{
		if (bDecimal)
			iCount++;
		if (*pPtr == '.')
			bDecimal = 1;
		pPtr++;
	}

	if (iPlaces > iCount)
	{
		if (!bDecimal)
			*pPtr++ = '.';
		while (iPlaces > iCount)
		{
			*pPtr++ = '0';
			iCount++;
		}
	}
	else if (iPlaces < iCount)
	{
		pPtr -= iCount - iPlaces;
		if (bRound)
		{
		}
	}
	else 
	{
	}
	*pPtr = '\0';

	return pStr;
}

Char *RemoveDecimal (Char *pStr)
{
	Char *pPtr;

	pPtr = pStr;
	while (*pPtr)
	{
		if (*pPtr == '.')
		{
			StrCopy(pPtr, (pPtr+1));
			return pStr;
		}
		pPtr++;
	}

	return pStr;
}

Char *AddDecimal (Char *pStr, Int16 iPlace)
{
	Char *pSrc, *pDest;
	Int16 iCount;

	pSrc = pStr;
	while (*pSrc == '0')
		pSrc++;
	iCount = StrLen(pSrc);

	if (iCount == 0)
	{
		// All zeros or blank make it zero.
		StrCopy(pStr, "0");
		return pStr;
	}

	if (iPlace == 0)
	{
		// Strip the leading zeros.
		if (pStr != pSrc)
			MemMove(pStr, pSrc, iCount + 1);
		return pStr;
	}

	if (iPlace < 0)
	{
		// Strip the leading zeros.
		if (pStr != pSrc)
			MemMove(pStr, pSrc, iCount + 1);
		pSrc = pStr + iCount;
		while (iPlace++ < 0)
			*pSrc++ = '0';
		*pSrc = '\0';
		return pStr;
	}
	
	if (iPlace >= iCount)
	{
		pDest = pStr + (iPlace - iCount) + 2;
		MemMove(pDest, pSrc, iCount+1);
		while (iPlace-- > iCount)
			*(--pDest) = '0';
		*(--pDest) = '.';
		*(--pDest) = '0';
	}
	else
	{
		// Strip the leading zeros.
		if (pStr != pSrc)
			MemMove(pStr, pSrc, iCount + 1);
		iPlace++;
		pSrc = pStr + iCount;
		pDest = pSrc + 1;
		while (iPlace--)
			*pDest-- = *pSrc--;
		*pDest = '.';
	}

	// Erase the trailing zeros.
	pSrc = pStr + StrLen(pStr) - 1;
	while (*pSrc == '0')
		pSrc--;
	// Leave one trailing zero.
	if (*pSrc == '.')
		pSrc++;
	*(++pSrc) = '\0';

	return pStr;
}

Boolean UpperGreater (Char *pUpper, Char *pLower)
{
	Int16 iUpperCount, iLowerCount;

	while ((*pUpper == '-') || (*pUpper == '0'))
		pUpper++;
	while ((*pLower == '-') || (*pLower == '0'))
		pLower++;
	iUpperCount = StrLen(pUpper);
	iLowerCount = StrLen(pLower);

	if (iUpperCount < iLowerCount)
	{
		return 0;
	}
	else if (iUpperCount == iLowerCount)
	{
		while (*pUpper && *pLower) 
		{
			if (*pUpper != *pLower)
				return (*pUpper > *pLower);
			*pUpper++;
			*pLower++;
		}
	}

	return 1;
}

Boolean IsZero (Char *pStr)
{
	while (*pStr)
	{
		if ((*pStr >= '1') && (*pStr <= '9'))
			return 0;
		pStr++;
	}

	return 1;
}

void AddStrings (Char *pUpper, Char *pLower, Char *pTotal, Int16 iDecimalPlaces)
{
	Int16 iUpperCount, iLowerCount;
	Char *pUpperPtr, *pLowerPtr, *pTotalPtr;
	Char *pUpperTemp, *pLowerTemp;
	Boolean bUpperNeg, bLowerNeg;

	// If the upper or lower is zero use the other value.
	if (IsZero(pUpper))
	{
		StrCopy(pTotal, pLower);
		return;
	}
	if (IsZero(pLower))
	{
		StrCopy(pTotal, pUpper);
		return;
	}

	// Create temp upper and lower values.
	pUpperTemp = (Char*)MemPtrNew(MATH_STRING);
	pLowerTemp = (Char*)MemPtrNew(MATH_STRING);
	StrCopy(pUpperTemp, pUpper);
	StrCopy(pLowerTemp, pLower);

	// Make both numbers the same decimal count.
	iUpperCount = DecimalCount(pUpperTemp);
	iLowerCount = DecimalCount(pLowerTemp);
	if (iUpperCount < iLowerCount)
		iUpperCount = iLowerCount;
	pUpperPtr = MakeDecimal(pUpperTemp, iUpperCount, true);
	pLowerPtr = MakeDecimal(pLowerTemp, iUpperCount, true);

	// Remove the decimal point
	RemoveDecimal(pUpperPtr);
	RemoveDecimal(pLowerPtr);

	// The larger number should be on top.
	if (!UpperGreater(pUpperPtr, pLowerPtr))
	{
		Char *pTemp = pUpperPtr;
		pUpperPtr = pLowerPtr;
		pLowerPtr = pTemp;
	}

	// Find the negative values.
	if (*pUpperPtr == '-')
	{
		pUpperPtr++;
		bUpperNeg = 1;
	}
	else
	{
		bUpperNeg = 0;
	}
	if (*pLowerPtr == '-')
	{
		pLowerPtr++;
		bLowerNeg = 1;
	}
	else
	{
		bLowerNeg = 0;
	}

	// Reverse the strings.
	StrRev(pUpperPtr);
	StrRev(pLowerPtr);
	pTotalPtr = pTotal;

	if (bUpperNeg == bLowerNeg)
	{
		Int16 iCarry = 0;

		// Do addition.
		while (*pUpperPtr)
		{
			iCarry += *pUpperPtr++ - '0';
			if (*pLowerPtr)
				iCarry += *pLowerPtr++ - '0';
			
			*pTotalPtr++ = (iCarry % 10) + '0';
			iCarry /= 10;
		}
		if (iCarry)
			*pTotalPtr++ = iCarry + '0';
	}
	else
	{
		Int16 iValue;

		// Do subtraction.
		while (*pUpperPtr)
		{
			iValue = *pUpperPtr++ - '0';
			if (*pLowerPtr)
				iValue -= *pLowerPtr++ - '0';
			if (iValue < 0)
			{
				Char *pPtr = pUpperPtr;
			
				while (*pPtr == '0')
				{
					*pPtr++ = '9';
				}
				*pPtr = *pPtr - 1;
				iValue += 10;
			}
			
			*pTotalPtr++ = iValue + '0';
		}
	}
	*pTotalPtr = '\0';
	AddDecimal(StrRev(pTotal), iUpperCount);

	// Add the neg sign back.
	if (bUpperNeg)
	{
		if (!IsZero(pTotal))
		{
			MemMove((pTotal+1), pTotal, StrLen(pTotal)+1);
			*pTotal = '-';
		}
	}

	MemPtrFree(pLowerTemp);
	MemPtrFree(pUpperTemp);
}

void SubtractStrings (Char *pUpper, Char *pLower, Char *pTotal, Int16 iDecimalPlaces)
{
	if (*pLower == '-')
	{
		AddStrings(pUpper, (pLower + 1), pTotal, iDecimalPlaces);
	}
	else
	{
		Char *pLowerTemp;

		// Create temp lower value.
		pLowerTemp = (Char*)MemPtrNew(MATH_STRING);
		StrCopy((pLowerTemp + 1), pLower);
		*pLowerTemp = '-';
		AddStrings(pUpper, pLowerTemp, pTotal, iDecimalPlaces);
		MemPtrFree(pLowerTemp);
	}
}

void MultiplyStrings (Char *pUpper, Char *pLower, Char *pTotal, Int16 iDecimalPlaces)
{
	Int16 iUpperCount, iCarry, i;
	Char *pUpperPtr, *pLowerPtr;
	Char *pUpperTempPtr, *pLowerTempPtr, *pTotalTempPtr;
	Char *pUpperTemp, *pLowerTemp, *pTotalTemp;
	Boolean bUpperNeg, bLowerNeg;

	// If the upper or lower is zero we do nothing.
	if (IsZero(pUpper) || IsZero(pLower))
	{
		StrCopy(pTotal, "0");
		return;
	}

	// Create temp upper and lower values.
	pUpperTemp = (Char*)MemPtrNew(MATH_STRING);
	pLowerTemp = (Char*)MemPtrNew(MATH_STRING);
	pTotalTemp = (Char*)MemPtrNew(MATH_STRING);
	StrCopy(pUpperTemp, pUpper);
	StrCopy(pLowerTemp, pLower);
	StrCopy(pTotal, "0");

	// Get the decimal count.
	iUpperCount = DecimalCount(pUpperTemp);
	iUpperCount += DecimalCount(pLowerTemp);

	// Remove the decimal point
	pUpperPtr = RemoveDecimal(pUpperTemp);
	pLowerPtr = RemoveDecimal(pLowerTemp);

	// Find the negative values.
	if (*pUpperPtr == '-')
	{
		pUpperPtr++;
		bUpperNeg = 1;
	}
	else
	{
		bUpperNeg = 0;
	}
	if (*pLowerPtr == '-')
	{
		pLowerPtr++;
		bLowerNeg = 1;
	}
	else
	{
		bLowerNeg = 0;
	}

	// Reverse the strings.
	StrRev(pUpperPtr);
	StrRev(pLowerPtr);

	if (iDecimalPlaces == -1)
		iDecimalPlaces = MAX_DECIMALS;

	// Do addition.
	pLowerTempPtr = pLowerPtr;
	while (*pLowerTempPtr)
	{
		if (*pLowerTempPtr == '0')
		{
			pLowerTempPtr++;
			continue;
		}

		iCarry = 0;
		pUpperTempPtr = pUpperPtr;
		pTotalTempPtr = pTotalTemp;
		for (i=0; i<(pLowerTempPtr - pLowerPtr); i++)
			*pTotalTempPtr++ = '0';
		while (*pUpperTempPtr)
		{
			iCarry = ((*pLowerTempPtr - '0') * (*pUpperTempPtr++ - '0')) + iCarry;
			*pTotalTempPtr++ = (iCarry % 10) + '0';
			iCarry /= 10;
		}
		if (iCarry)
			*pTotalTempPtr++ = iCarry + '0';
		*pTotalTempPtr = '\0';
		AddStrings(StrRev(pTotalTemp), pTotal, pTotal, -1);
		pLowerTempPtr++;
	}
	AddDecimal(pTotal, iUpperCount);
	if (iDecimalPlaces < iUpperCount)
		MakeDecimal(pTotal, iDecimalPlaces, true);

	// Add the neg sign back.
	if (bUpperNeg != bLowerNeg)
	{
		if (!IsZero(pTotal))
		{
			MemMove((pTotal+1), pTotal, StrLen(pTotal)+1);
			*pTotal = '-';
		}
	}

	MemPtrFree(pTotalTemp);
	MemPtrFree(pLowerTemp);
	MemPtrFree(pUpperTemp);
}

void DivideStrings (Char *pUpper, Char *pLower, Char *pTotal, Int16 iDecimalPlaces)
{
	Int16 iUpperCount, i;
	Char *pUpperPtr, *pLowerPtr, *pTotalPtr;
	Char *pUpperTempPtr;
	Char *pUpperTemp, *pLowerTemp;
	Boolean bUpperNeg, bLowerNeg;
	Char *pDivisors [9];

	// If the upper or lower is zero we do nothing.
	if (IsZero(pUpper) || IsZero(pLower))
	{
		StrCopy(pTotal, "0");
		return;
	}

	// Create temp upper and lower values.
	pUpperTemp = (Char*)MemPtrNew(MATH_STRING);
	pLowerTemp = (Char*)MemPtrNew(MATH_STRING);
	for (i=0; i<9; i++)
		pDivisors [i] = (Char*)MemPtrNew(MATH_STRING);

	StrCopy(pUpperTemp, pUpper);
	StrCopy(pLowerTemp, pLower);

	// Get the decimal count.
	iUpperCount = DecimalCount(pUpperTemp);
	iUpperCount -= DecimalCount(pLowerTemp);

	// Remove the decimal point
	pUpperPtr = RemoveDecimal(pUpperTemp);
	pLowerPtr = RemoveDecimal(pLowerTemp);

	// Find the negative values.
	if (*pUpperPtr == '-')
	{
		pUpperPtr++;
		bUpperNeg = 1;
	}
	else
	{
		bUpperNeg = 0;
	}
	if (*pLowerPtr == '-')
	{
		pLowerPtr++;
		bLowerNeg = 1;
	}
	else
	{
		bLowerNeg = 0;
	}

	for (i=0; i<9; i++)
	{
		Char szBuffer [3];

		*szBuffer = i + '0' + 1;
		*(szBuffer+1) = '\0';
		MultiplyStrings(pLowerPtr, szBuffer, pDivisors [i], -1);
	}

	if (iDecimalPlaces == -1)
		iDecimalPlaces = MAX_DECIMALS;

	// Do division.
	pUpperTempPtr = pUpperPtr;
	pLowerPtr = pLowerTemp;
	pTotalPtr = pTotal;
	while (*pUpperTempPtr || !IsZero(pLowerTemp))
	{
		if (*pUpperTempPtr)
		{
			*pLowerPtr++ = *pUpperTempPtr;
		}
		else
		{
			// If we have exceded this count, break;
			if (iUpperCount >= iDecimalPlaces)
				break;
			*pLowerPtr++ = '0';
			iUpperCount++;
		}

		*pLowerPtr = '\0';
		*pTotalPtr = '0';
		if (UpperGreater(pLowerTemp, pDivisors [0]))
		{
			for (i=8; i>=0; i--)
			{
				if (UpperGreater(pLowerTemp, pDivisors [i]))
				{
					*pTotalPtr = '0' + i + 1;
					SubtractStrings(pLowerTemp, pDivisors [i], pLowerTemp, -1);
					pLowerPtr = pLowerTemp + StrLen(pLowerTemp);
					break;
				}
			}
		}
		pTotalPtr++;
		if (*pUpperTempPtr)
			pUpperTempPtr++;
	}
	*pTotalPtr = '\0';
	AddDecimal(pTotal, iUpperCount);
	if (iDecimalPlaces < iUpperCount)
		MakeDecimal(pTotal, iDecimalPlaces, true);

	// Add the neg sign back.
	if (bUpperNeg != bLowerNeg)
	{
		if (!IsZero(pTotal))
		{
			MemMove((pTotal+1), pTotal, StrLen(pTotal)+1);
			*pTotal = '-';
		}
	}

	for (i=0; i<9; i++)
		MemPtrFree(pDivisors [i]);
	MemPtrFree(pLowerTemp);
	MemPtrFree(pUpperTemp);
}

void ModStrings (Char *pUpper, Char *pLower, Char *pTotal)
{
	Char *pUpperPtr, *pLowerPtr;
	Char *pUpperTempPtr;
	Char *pUpperTemp, *pLowerTemp;
	Boolean bUpperNeg, bLowerNeg;
	Char *pDivisors [9];
	Int16 i;

	// If the upper or lower is zero we do nothing.
	if (IsZero(pUpper) || IsZero(pLower) || !UpperGreater(pUpper, pLower))
	{
		StrCopy(pTotal, ((IsZero(pUpper)) ? "0" : pUpper));
		return;
	}

	// Create temp upper and lower values.
	pUpperTemp = (Char*)MemPtrNew(MATH_STRING);
	pLowerTemp = (Char*)MemPtrNew(MATH_STRING);
	for (i=0; i<9; i++)
		pDivisors [i] = (Char*)MemPtrNew(MATH_STRING);

	StrCopy(pUpperTemp, pUpper);
	StrCopy(pLowerTemp, pLower);

	// Remove the decimal point
	pUpperPtr = GetWhole(pUpperTemp, pUpperTemp);
	pLowerPtr = GetWhole(pLowerTemp, pLowerTemp);

	// Find the negative values.
	if (*pUpperPtr == '-')
	{
		pUpperPtr++;
		bUpperNeg = 1;
	}
	else
	{
		bUpperNeg = 0;
	}
	if (*pLowerPtr == '-')
	{
		pLowerPtr++;
		bLowerNeg = 1;
	}
	else
	{
		bLowerNeg = 0;
	}

	for (i=0; i<9; i++)
	{
		Char szBuffer [3];

		*szBuffer = i + '0' + 1;
		*(szBuffer+1) = '\0';
		MultiplyStrings(pLowerPtr, szBuffer, pDivisors [i], -1);
	}

	// Do division.
	pUpperTempPtr = pUpperPtr;
	pLowerPtr = pLowerTemp;
	while (*pUpperTempPtr || StrCompare(pLowerTemp, "0"))
	{
		if (!*pUpperTempPtr)
			break;
		*pLowerPtr++ = *pUpperTempPtr;
		*pLowerPtr = '\0';
		if (UpperGreater(pLowerTemp, pDivisors [0]))
		{
			for (i=8; i>=0; i--)
			{
				if (UpperGreater(pLowerTemp, pDivisors [i]))
				{
					SubtractStrings(pLowerTemp, pDivisors [i], pLowerTemp, -1);
					pLowerPtr = pLowerTemp + StrLen(pLowerTemp);
					break;
				}
			}
		}
		if (*pUpperTempPtr)
			pUpperTempPtr++;
	}
	// Copy the remainer.
	StrCopy(pTotal, pLowerTemp);

	// Add the neg sign back.
	if (bUpperNeg != bLowerNeg)
	{
		if (!IsZero(pTotal))
		{
			MemMove((pTotal+1), pTotal, StrLen(pTotal)+1);
			*pTotal = '-';
		}
	}

	for (i=0; i<9; i++)
		MemPtrFree(pDivisors [i]);
	MemPtrFree(pLowerTemp);
	MemPtrFree(pUpperTemp);
}

Boolean IsNegative (Char *pStr)
{
	return (*pStr == '-');
}

void Negate (Char *pStr)
{
	if (*pStr == '-')
	{
		StrCopy(pStr, (pStr+1));
	}
	else
	{
		MemMove((pStr+1), pStr, StrLen(pStr)+1);
		*pStr = '-';
	}
}

Char *GetWhole (Char *pStr, Char *pWhole)
{
	Char *pPtr = pStr;

	while (*pPtr)
	{
		if (*pPtr == '.')
			break;
		pPtr++;
	}
	MemMove(pWhole, pStr, (pPtr - pStr));
	pWhole [pPtr - pStr] = '\0';

	return pWhole;
}
