#include "StdInc.h"
#include "DatabaseInfo.h"

//Char szBuffer1 [32], szBuffer2 [32], szBuffer3 [32];
//StrIToA(szBuffer1, iIndex);
//StrIToA(szBuffer2, DmNumRecords(pRecordInfo->hDatabase));
//FrmCustomAlert(4003, szBuffer1, szBuffer2, "");

#define RECORDINDEX_NONE	((UInt16)-1)

Int16 RecordInfoSort (void *pRecord1, void *pRecord2, Int16 iOther, SortRecordInfoPtr pSortInfo1, SortRecordInfoPtr pSortInfo2, MemHandle hAppInfo)
{
	return MemCompare(pRecord1, pRecord2, (sizeof(Int32) * 2));
}

Boolean InitializeRecordInfo (RecordInfoPtr pRecordInfo)
{
	MemSet(pRecordInfo, sizeof(RecordInfo), '\0');
	return true;
}

Boolean FindIndexRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->hDatabase == 0)
		return false;

	if (pRecordInfo->vId == RECORDID_INDEX)
	{
		if (DmNumRecords(pRecordInfo->hDatabase) < pRecordInfo->vIndex)
			pRecordInfo->vIndex = RECORDINDEX_NONE;
	}
	else
	{
		UInt32 lTempRecord [2];

		// Make a temporary record.
		lTempRecord [0] = pRecordInfo->vId;
		lTempRecord [1] = pRecordInfo->vSubId;

		// Find the insertion point of the record.
		pRecordInfo->vIndex = DmFindSortPosition(pRecordInfo->hDatabase, &lTempRecord, 0, RecordInfoSort, 0) - 1;

		if (pRecordInfo->vIndex != RECORDINDEX_NONE)
		{
			MemHandle hRecord;
			Char *pRecord;

			hRecord = DmQueryRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex);
			pRecord = MemHandleLock(hRecord);
			if (MemCmp(pRecord, &lTempRecord, (sizeof(Int32) * 2)) != 0)
				pRecordInfo->vIndex = RECORDINDEX_NONE;
			MemHandleUnlock(hRecord);
		}
	}

	return (pRecordInfo->vIndex != RECORDINDEX_NONE);
}

Boolean CreateDatabaseRecordInfo (RecordInfoPtr pRecordInfo, UInt32 ulAppId, UInt32 ulDatabaseId)
{
	Char szDatabaseName [10];

	MemMove(szDatabaseName, &ulAppId, 4);
	*(szDatabaseName+4) = '-';
	MemMove((szDatabaseName+5), &ulDatabaseId, 4);
	*(szDatabaseName+9) = '\0';

	if (DmCreateDatabase(0, szDatabaseName, ulAppId, ulDatabaseId, false)) 
		return false;

	return OpenDatabaseRecordInfo(pRecordInfo, ulAppId, ulDatabaseId);
}

Boolean OpenDatabaseRecordInfo (RecordInfoPtr pRecordInfo, UInt32 ulAppId, UInt32 ulDatabaseId)
{
	pRecordInfo->hDatabase = DmOpenDatabaseByTypeCreator(ulDatabaseId, ulAppId, dmModeReadWrite);
	
	return (pRecordInfo->hDatabase == 0) ? false : true;
}

Boolean CloseDatabaseRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->hDatabase)
	{
		// Close the database
//		DmResetRecordStates(pRecordInfo->hDatabase); // This crashes the app.
		DmCloseDatabase(pRecordInfo->hDatabase);
		
		return true;
	}

	return false;
}

Boolean DeleteDatabaseRecordInfo (UInt32 ulAppId, UInt32 ulDatabaseId)
{
	Char szDatabaseName [10];
	LocalID vLocalId;

	MemMove(szDatabaseName, &ulAppId, 4);
	*(szDatabaseName+4) = '-';
	MemMove((szDatabaseName+5), &ulDatabaseId, 4);
	*(szDatabaseName+9) = '\0';

	vLocalId = DmFindDatabase(0, szDatabaseName);
	if (vLocalId == 0)
		return false;
	if (DmDeleteDatabase(0, vLocalId))
		return false;

	return true;
}

Boolean SetAttributesDatabaseRecordInfo (RecordInfoPtr pRecordInfo, UInt16 uAttributes, DbAttribType eDbAttribType)
{
	LocalID vLocalID;
	UInt16 uOldAttributes, uCardNumber;

	if (pRecordInfo->hDatabase == 0)
		return false;

	if (DmOpenDatabaseInfo(pRecordInfo->hDatabase, &vLocalID, NULL, NULL, &uCardNumber, NULL) != 0)
		return false;

	if (DmDatabaseInfo(uCardNumber, vLocalID, NULL, &uOldAttributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return false;

	switch (eDbAttribType)
	{
	case DbAttribTypeRemove:
		uOldAttributes &= ~uAttributes;
		break;
	case DbAttribTypeOverwrite:
		uOldAttributes = uAttributes;
		break;
	default:  // default is DbAttribTypeAdd.
		uOldAttributes |= uAttributes;
	}

	if (DmSetDatabaseInfo(uCardNumber, vLocalID, NULL, &uOldAttributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return false;

	return true;
}

Boolean SetNameDatabaseRecordInfo (RecordInfoPtr pRecordInfo, Char *pName)
{
	LocalID vLocalID;
	UInt16 uCardNumber;
	Char szName [dmDBNameLength];
	Int16 iNameSize;

	if ((pRecordInfo->hDatabase == 0) || (pName == 0))
		return false;

	iNameSize = StrLen(pName);
	if (iNameSize == 0)
		return false;

	// Truncate the name if needed.
	MemMove(szName, pName, ((iNameSize < dmDBNameLength) ? iNameSize : dmDBNameLength-1));
	szName[dmDBNameLength-1] = '\0';

	if (DmOpenDatabaseInfo(pRecordInfo->hDatabase, &vLocalID, NULL, NULL, &uCardNumber, NULL) != 0)
		return false;

	if (DmSetDatabaseInfo(uCardNumber, vLocalID, szName, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) != 0)
		return false;

	return true;
}

Boolean ChangeIdRecordInfo (RecordInfoPtr pRecordInfo, Int32 lNewId, Int32 lNewSubId)
{
	MemHandle hRecord;
	MemPtr pRecord;

	if (pRecordInfo->hDatabase) 
	{
		// Try to find the record.
		if (FindIndexRecordInfo(pRecordInfo) == true)
		{
			hRecord = DmGetRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex);
			pRecord = MemHandleLock(hRecord);

			// Write the record id
			if (lNewId != RECORDID_NONE)
				DmWrite(pRecord, 0, &lNewId, 4);
			// Write the record sub id
			if (lNewSubId != RECORDID_NONE)
				DmWrite(pRecord, 4, &lNewSubId, 4);
			// Unlock the record.
			MemHandleUnlock(hRecord); 
			DmReleaseRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex, true);

			return true;
		}
	}

	return false;
}

Boolean ReadRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->hDatabase)
	{
		// Try to find the record.
		if (FindIndexRecordInfo (pRecordInfo) == true)
		{
			// Get the handle and memory of the record.
			pRecordInfo->vDatabase.hRecord = DmGetRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex);
			pRecordInfo->vDatabase.pRecord = MemHandleLock(pRecordInfo->vDatabase.hRecord);

			// We have the correct record set the other info.
			pRecordInfo->vDatabase.vLength = MemHandleSize(pRecordInfo->vDatabase.hRecord) - 8;
			pRecordInfo->vDatabase.pRecord = ((Char*)pRecordInfo->vDatabase.pRecord) + 8;

			return true;
		}
	}

	return false;
}

Boolean WriteRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->hDatabase)
	{
		MemHandle pRecord;

		// Is the record already open?
		if (pRecordInfo->vDatabase.hRecord == 0)
		{
			// Try to find the record.
			if (FindIndexRecordInfo (pRecordInfo) == false)
			{
				return false;
			}

			// Get the handle and memory of the record.
			pRecordInfo->vDatabase.hRecord = DmGetRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex);
			pRecordInfo->vDatabase.pRecord = MemHandleLock(pRecordInfo->vDatabase.hRecord);

			// We have the correct record set the other info.
			pRecordInfo->vDatabase.vLength = MemHandleSize(pRecordInfo->vDatabase.hRecord) - 8;
			pRecordInfo->vDatabase.pRecord = ((Char*)pRecordInfo->vDatabase.pRecord) + 8;
		}

		// Write out the data at the the offset
		if (pRecordInfo->vInput.pRecord != 0)
			DmWrite(pRecordInfo->vDatabase.pRecord, pRecordInfo->vInput.vOffset + 8, pRecordInfo->vInput.pRecord, pRecordInfo->vInput.vLength);

		return true;
	}

	return false;
}

Boolean InsertRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->hDatabase)
	{
		UInt16 iIndex;

		if (pRecordInfo->vId == RECORDID_INDEX)
		{
			if (DmNumRecords(pRecordInfo->hDatabase) < pRecordInfo->vIndex)
			{
				return false;
			}
		}
		else
		{
			Int32 lTempRecord [2];

			// Make a temporary record.
			lTempRecord [0] = pRecordInfo->vId;
			lTempRecord [1] = pRecordInfo->vSubId;

			// Find the insertion point of the record.
			pRecordInfo->vIndex = DmFindSortPosition(pRecordInfo->hDatabase, &lTempRecord, 0, RecordInfoSort, 0);
		}

		// Write the new record.
		pRecordInfo->vDatabase.hRecord = DmNewRecord(pRecordInfo->hDatabase, &pRecordInfo->vIndex, (pRecordInfo->vInput.vLength + 8));
		pRecordInfo->vDatabase.pRecord = MemHandleLock(pRecordInfo->vDatabase.hRecord);
		pRecordInfo->vDatabase.vLength = pRecordInfo->vInput.vLength;

		// Write the record id
		DmWrite(pRecordInfo->vDatabase.pRecord, 0, &pRecordInfo->vId, 4);
		// Write the record sub id
		DmWrite(pRecordInfo->vDatabase.pRecord, 4, &pRecordInfo->vSubId, 4);
		// Write the record data
		if (pRecordInfo->vInput.pRecord != 0)
			DmWrite(pRecordInfo->vDatabase.pRecord, 8, pRecordInfo->vInput.pRecord, pRecordInfo->vInput.vLength);

		return true;
	}

	return false;
}

Boolean OverwriteRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (DeleteRecordInfo(pRecordInfo) == false)
		return false;
	if (InsertRecordInfo(pRecordInfo) == false)
		return false;

	return true;
}

Boolean CloseRecordInfo (RecordInfoPtr pRecordInfo)
{
	if (pRecordInfo->vDatabase.hRecord != 0)
	{
		MemHandleUnlock(pRecordInfo->vDatabase.hRecord); 
		DmReleaseRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex, false);
		
		pRecordInfo->vDatabase.hRecord = 0;
		pRecordInfo->vDatabase.pRecord = 0;
	}

	return true;
}

Boolean DeleteRecordInfo (RecordInfoPtr pRecordInfo)
{
	MemHandle hRecord;
	MemPtr pRecord;

	if (pRecordInfo->hDatabase) 
	{
		// If RECORDID_NONE we delete all of the ID type records.
		if (pRecordInfo->vSubId == RECORDID_NONE)
		{
			Int16 iCount, i;
			UInt32 vId;

			// Get the total records.
			iCount = DmNumRecords(pRecordInfo->hDatabase);
			for (i=iCount-1; i>=0; i--)
			{
				// Get the id of the record.
				hRecord = DmQueryRecord(pRecordInfo->hDatabase, i);
				pRecord = MemHandleLock(hRecord);
				vId = *((UInt32*)pRecord);
				MemHandleUnlock(hRecord); 

				if (pRecordInfo->vId == vId)
					DmRemoveRecord(pRecordInfo->hDatabase, i);
			}
		}
		else
		{
			UInt16 iIndex;

			// Try to find the record.
			if (FindIndexRecordInfo(pRecordInfo) == true)
			{
				DmRemoveRecord(pRecordInfo->hDatabase, pRecordInfo->vIndex);
			}
		}
	}

	return true;
}

UInt16 GetCountRecordInfo (RecordInfoPtr pRecordInfo)
{
	UInt16 iFoundCount;
	MemHandle hRecord;
	MemPtr pRecord;
	
	// Default the count.
	iFoundCount = 0;
	if (pRecordInfo->hDatabase) 
	{
		Int16 iCount, i;

		// Get the total records.
		iCount = DmNumRecords(pRecordInfo->hDatabase);
		if (pRecordInfo->vId == RECORDID_NONE)
		{
			iFoundCount = iCount;
		}
		else
		{
			for (i=0; i<iCount; i++)
			{
				hRecord = DmGetRecord(pRecordInfo->hDatabase, i);
				pRecord = MemHandleLock(hRecord);

				// Is this the record we are looking for?
				if ((((UInt32*)pRecord) [0] == pRecordInfo->vId))
				{
					iFoundCount++;
				}

				// Unlock the record.
				MemHandleUnlock(hRecord); 
				DmReleaseRecord(pRecordInfo->hDatabase, i, false);
			}
		}
	}

	return iFoundCount;
}
