#include "StdInc.h"
#include "Misc.h"

Char *StrLToA (Char *lpStr, Int32 lNum)
{
    Boolean bNeg=false;
    Char *ptr1, *ptr2;
    Char ch;

    if (lNum < 0)
    {
        bNeg = true;
        lNum *= -1;
    }

    ptr1 = lpStr;
    ptr2 = lpStr;
    do
    {
        *ptr1++ = (lNum % 10) + '0';
        lNum /= 10;
    } while (lNum);
    if (bNeg)
        *ptr1++ = '-';
    *ptr1-- = '\0';
    while (ptr1 > ptr2)
    {
        ch = *ptr1;
        *ptr1-- = *ptr2;
        *ptr2++ = ch;
    }

    return lpStr;
}

Char *StrLToAComma (Char *lpStr, Int32 lNum)
{
    Boolean bNeg=false;
    Char *ptr1, *ptr2;
    Char ch;
	Int16 iTemp;

    if (lNum < 0)
    {
        bNeg = true;
        lNum *= -1;
    }

	iTemp = 1;
    ptr1 = lpStr;
    ptr2 = lpStr;
    do
    {
        *ptr1++ = (lNum % 10) + '0';
        lNum /= 10;
		if (lNum && ((iTemp++ % 3) == 0)) 
			*ptr1++ = ',';
    } while (lNum);
    if (bNeg)
        *ptr1++ = '-';
    *ptr1-- = '\0';
    while (ptr1 > ptr2)
    {
        ch = *ptr1;
        *ptr1-- = *ptr2;
        *ptr2++ = ch;
    }

    return lpStr;
}

Int32 StrAToL (Char *lpStr)
{
    Int32 lResult = 0;
    Boolean bNeg = false;

    if (*lpStr == '+')
        lpStr++;
    else if (*lpStr == '-')
    {
        lpStr++;
        bNeg = true;
    }
    while (*lpStr)
    {
        lResult *= 10;
        lResult += *lpStr++ - '0';
    }
    if (bNeg)
        lResult *= -1;

    return lResult;
}

Char *StrRev (Char *pStr)
{
	Char *pStart, *pEnd;
	Char ch;

	pStart = pStr;
	pEnd = (pStr + StrLen(pStr)) - 1;
	while (pStart < pEnd)
	{
		ch = *pStart;
		*pStart++ = *pEnd;
		*pEnd-- = ch;
	}

	return pStr;
}

Int16 MemCompare (Char *ptr1, Char *ptr2, Int16 iCount)
{
	while (iCount-- > 0)
	{
		if (*ptr1 != *ptr2)
			return (*ptr1 - *ptr2);
		ptr1++;
		ptr2++;
	}

	return 0;
}

void QShuffle (UInt8 *base, Int16 numOfElements, Int16 width)
{
	UInt8 *ptr1, *ptr2;
    UInt8 ch;
	Int16 i, pos;

	while (numOfElements)
	{
		pos = SysRandom(0) % numOfElements--;
		if (pos == numOfElements) continue;
		ptr1 = (UInt8*)(base + pos * width);
		ptr2 = (UInt8*)(base + numOfElements * width);
		i = width;
		while (i--)
		{
			ch = *ptr1;
			*ptr1++ = *ptr2;
			*ptr2++ = ch;
		}
	}
}

void InitPopup (FormPtr pForm, UInt16 wTriggerId, UInt16 wListId, Int16 iCurrent)
{
	ControlPtr pTrigger;
	ListPtr pList;
	Char *pText = 0;

	pTrigger = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wTriggerId));
	pList = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wListId));
	if ((iCurrent >= 0) && (iCurrent < LstGetNumberOfItems(pList)))
	{
		LstSetSelection(pList, iCurrent);
		LstMakeItemVisible(pList, iCurrent);
		pText = LstGetSelectionText(pList, iCurrent);
	}
	CtlSetLabel(pTrigger, (pText == 0) ? "" : pText);
}

void InitDynamicPopup (FormPtr pForm, UInt16 wTriggerId, UInt16 wListId, Int16 iCurrent, Char **pItems, Int16 iSize)
{
	ControlPtr pTrigger;
	ListPtr pList;
	Char *pText = 0;

	pTrigger = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wTriggerId));
	pList = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wListId));

	LstSetHeight(pList, (iSize > 10) ? 10 : iSize);
	LstSetListChoices(pList, pItems, iSize);

	if ((iCurrent >= 0) && (iCurrent < LstGetNumberOfItems(pList)))
	{
		LstSetSelection(pList, iCurrent);
		LstMakeItemVisible(pList, iCurrent);
		if (pItems)
		{
			pText = LstGetSelectionText(pList, iCurrent);
		}
	}
	CtlSetLabel(pTrigger, (pText == 0) ? "" : pText);
}

MemHandle SetField (FormPtr pForm, UInt16 wFieldId, UInt32 ulMaxChars, Char *pText)
{
    FieldPtr fpField;
    MemHandle hNewHandle, hOldHandle;
	Char *pNewText, *pOldText;
	UInt32 ulOldSize;
	Boolean bUseOld = false;

    fpField = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wFieldId));
    hNewHandle = MemHandleNew(ulMaxChars+1);
    if (hNewHandle)
    {
		pNewText = MemHandleLock(hNewHandle);
		hOldHandle = (MemHandle)FldGetTextHandle(fpField);
        if (pText)
        {
			pOldText = pText;
        }
        else if (hOldHandle)
		{
			pOldText = MemHandleLock(hOldHandle);
			bUseOld = true;
		}
		else
		{
			pOldText = "";
		}

		ulOldSize = StrLen(pOldText);
		if (ulOldSize > ulMaxChars)
			ulOldSize = ulMaxChars;

		MemMove(pNewText, pOldText, ulOldSize);
		pNewText [ulOldSize] = '\0';

		MemHandleUnlock(hNewHandle);
        FldSetMaxChars(fpField, ulMaxChars);
        FldSetTextHandle(fpField, (MemHandle)hNewHandle);

        if (hOldHandle)
		{
			if (bUseOld)
				MemHandleUnlock(hOldHandle);
            MemHandleFree(hOldHandle);
		}
    }

    return hNewHandle;
}

void DrawCharacters (Char *chars, UInt16 len, RectanglePtr pRect, UInt8 type, Boolean show)
{
	Int16 x, y, w, h;
	RectangleType vClipRect;

	w = FntCharsWidth(chars, len);
	h = FntLineHeight();

	switch (type)
	{
	case DC_RIGHT:
		x = (pRect->topLeft.x + pRect->extent.x) - w;
		break;
	case DC_CENTER:
		x = pRect->topLeft.x + (((pRect->extent.x - w) + 1) / 2);
		break;
	default:
		x = pRect->topLeft.x + 1;
	}

	// Center vertically
	y = pRect->topLeft.y + ((pRect->extent.y - h) / 2);

	WinGetClip(&vClipRect);
	WinSetClip(pRect);
	if (show)
		WinDrawChars(chars, len, x, y);
	else
		WinEraseChars(chars, len, x, y);
	WinSetClip(&vClipRect);
}

void SetCheck (FormPtr pForm, UInt16 wFieldId, Boolean bCheckState)
{
	CtlSetValue(FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wFieldId)), (short)bCheckState);
}

Boolean GetCheck (FormPtr pForm, UInt16 wFieldId)
{
	return (Boolean)CtlGetValue(FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wFieldId)));
}

void SetPushButton (FormPtr pForm, UInt16 wStartFieldId, UInt16 wEndFieldId, UInt16 wFieldId)
{
	UInt16 w;
	Boolean bCheck, bChecked;

	for (w=wStartFieldId; w<=wEndFieldId; w++)
	{
		if ((w != wFieldId) && GetCheck(pForm, w))
		{
			SetCheck(pForm, w, false);
		}
	}
	SetCheck(pForm, wFieldId, true);
}

UInt16 GetPushButton (FormPtr pForm, UInt16 wStartFieldId, UInt16 wEndFieldId)
{
	UInt16 w;

	for (w=wStartFieldId; w<=wEndFieldId; w++)
	{
		if (GetCheck(pForm, w))
		{
			return w;
		}
	}

	return 0;
}

inline void InflateRect (RectanglePtr pRect, Int16 x, Int16 y)
{
	pRect->topLeft.x -= x;
	pRect->topLeft.y -= y;
	pRect->extent.x += x + x;
	pRect->extent.y += y + y;
}

inline void AutoShiftField (FormPtr pForm, UInt16 wId, Boolean bAutoShift)
{
	FieldPtr pField;
	FieldAttrType vFieldAttr;

	pField = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wId));
	FldGetAttributes (pField, &vFieldAttr);
	vFieldAttr.autoShift = bAutoShift;
	FldSetAttributes (pField, &vFieldAttr);
}

Boolean IsRomVersion (UInt32 vVersion)
{
	UInt32 vRomVersion;

	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &vRomVersion);

	return (vRomVersion >= vVersion);
}

void RefreshForm ()
{
	EventType vEvent;

	// Make a frmUpdateEvent event.
	MemSet(&vEvent, sizeof(EventType), 0);
	vEvent.eType = frmUpdateEvent;
	vEvent.data.frmUpdate.formID = FrmGetActiveFormID();

	// Add it to the queue.
	EvtAddEventToQueue(&vEvent);
}
