#include "StdInc.h"

#define TEXT_X      8
#define TEXT_Y      17
#define TEXT_WIDTH  139

// Structs 
typedef struct _tagNumbersMemberVars
{
	Int32 lMin;
	Int32 lMax;
	Int32 lValue;
	Int32 lDefault;
	Boolean bAddedDigit;
} NumbersMemberVars;
typedef NumbersMemberVars *NumbersMemberVarsPtr;

// Vars
NumbersMemberVarsPtr g_pNumberMemberVars;

// Funcs
Boolean NumbersProc(EventPtr event);
void AddDigit (long lNumber);
void DeleteDigit ();
void SwitchSign ();
void ClearNumber();
void DrawNumber ();
void DrawNumbers (FormPtr pForm);

UInt16 DoModalNumbers (Char *pTitle, Int32 *pValue, Int32 lMin, Int32 lMax)
{
	static Boolean bInit = false;
	FormPtr pForm, pOldForm;
	UInt16 wResult;
	Char *pOldTitle;

	wResult = 0;
	pOldForm = FrmGetActiveForm();
	if (pForm = FrmInitForm(IDF_NUMBERS))
	{
		// Pre form stuff.
		FrmSetActiveForm(pForm);
		FrmDrawForm(pForm);

		// Do the every time initialization 
		//  of the Sound dialog here.
		g_pNumberMemberVars = MemPtrNew(sizeof(NumbersMemberVars));
		if (g_pNumberMemberVars)
		{
			g_pNumberMemberVars->lValue = *pValue;
			g_pNumberMemberVars->lDefault = *pValue;
			g_pNumberMemberVars->lMin = lMin;
			g_pNumberMemberVars->lMax = lMax;
			g_pNumberMemberVars->bAddedDigit = false;

			if (pTitle)
			{
				pOldTitle = (Char*)FrmGetTitle(pForm);
				FrmSetTitle(pForm, pTitle);
			}

			// Draw the other stuff.
			DrawNumbers(pForm);

			// Show the dialog.
			FrmSetEventHandler(pForm, (FormEventHandlerPtr) NumbersProc);
			wResult = FrmDoDialog(pForm);

			if (pTitle)
				FrmSetTitle(pForm, pOldTitle);

			if (wResult == IDC_NUMBERSOK)
			{
				*pValue = g_pNumberMemberVars->lValue;
			}
			MemPtrFree(g_pNumberMemberVars);
		}

		// We now return to our other window.
		if (pOldForm)
			FrmSetActiveForm(pOldForm);
		FrmDeleteForm(pForm);
	}

	return wResult;
}

Boolean NumbersProc(EventPtr event)
{
	FormPtr pForm = FrmGetActiveForm();
	Boolean bHandled = false;

    switch (event->eType) 
    {
	case frmUpdateEvent:
	{
		FrmDrawForm(pForm);
		DrawNumbers(pForm);
		bHandled = true;
		break;
	}

    case ctlSelectEvent:  // A control button was pressed and released.
	{
        bHandled = true;
        switch (event->data.ctlEnter.controlID)
        {
        case IDC_0:
        case IDC_1:
        case IDC_2:
        case IDC_3:
        case IDC_4:
        case IDC_5:
        case IDC_6:
        case IDC_7:
        case IDC_8:
        case IDC_9:
			if (g_pNumberMemberVars->bAddedDigit == false)
			{
				// If this is the first digit clear it.
				g_pNumberMemberVars->bAddedDigit = true;
				ClearNumber();
			}
			AddDigit(event->data.ctlEnter.controlID - IDC_0);
			break;
		case IDC_CLEAR:
			ClearNumber();
			break;
		case IDC_BACKSPACE:
			if (g_pNumberMemberVars->bAddedDigit == false)
			{
				// If this is the first digit clear it.
				g_pNumberMemberVars->bAddedDigit = true;
				ClearNumber();
			}
			DeleteDigit();
			break;
		case IDC_PLUSMINUS:
			if (g_pNumberMemberVars->bAddedDigit == false)
			{
				// If this is the first digit clear it.
				g_pNumberMemberVars->bAddedDigit = true;
				ClearNumber();
			}
			SwitchSign();
			break;
        case IDC_NUMBERSCANCEL:
			g_pNumberMemberVars->lValue = g_pNumberMemberVars->lDefault;
			bHandled = false;
            break;
        case IDC_NUMBERSOK:
		{
			if ((g_pNumberMemberVars->lValue < g_pNumberMemberVars->lMin) || 
				(g_pNumberMemberVars->lValue > g_pNumberMemberVars->lMax))
			{
				Char szMinRange [32];
				Char szMaxRange [32];

				StrLToAComma(szMinRange, g_pNumberMemberVars->lMin);
				StrLToAComma(szMaxRange, g_pNumberMemberVars->lMax);
				FrmCustomAlert(IDA_NUMBERSERROR, szMinRange, szMaxRange, "");
				bHandled = true;
				break;
			}
			bHandled = false;
            break;
		}
        break;

		}
	}

	default:;

    }

	return bHandled;
}

void AddDigit (long lNumber)
{
	long lTemp;

	lTemp = g_pNumberMemberVars->lValue * 10;
	if (lTemp < 0)
		lTemp -= lNumber;
	else
		lTemp += lNumber;

	if ((g_pNumberMemberVars->lValue < 100000000) && (g_pNumberMemberVars->lValue > -100000000))
	{
		g_pNumberMemberVars->lValue = lTemp;
		DrawNumber();
	}
}

void DeleteDigit ()
{
	g_pNumberMemberVars->lValue /= 10;
	DrawNumber();
}

void SwitchSign ()
{
	long lTemp;

	lTemp = g_pNumberMemberVars->lValue * -1;
	g_pNumberMemberVars->lValue = lTemp;
	DrawNumber();
}

void ClearNumber()
{
	g_pNumberMemberVars->lValue = 0;
	DrawNumber();
}

void DrawNumber ()
{
    Int16 iCurrentX=TEXT_X, iCurrentY=TEXT_Y, iTemp, i;
    Int16 iCharHeight;
    Char szText [50];
    RectangleType rtRect;
	short sWidth;
    FontID fiCurrent;

    fiCurrent = FntSetFont(6);
    iCharHeight = FntCharHeight();

    rtRect.topLeft.x = iCurrentX;
    rtRect.topLeft.y = iCurrentY;
    rtRect.extent.x = TEXT_WIDTH;
    rtRect.extent.y = iCharHeight + 1;

	WinDrawRectangleFrame(rectangleFrame, &rtRect);

    rtRect.topLeft.x++;
    rtRect.topLeft.y++;
    rtRect.extent.x -= 8;
    rtRect.extent.y -= 2;

	WinEraseRectangle(&rtRect, 0);

    StrLToAComma(szText, g_pNumberMemberVars->lValue);
	sWidth = FntLineWidth(szText, StrLen(szText));
    WinDrawChars(szText, StrLen(szText), rtRect.topLeft.x + (rtRect.extent.x - sWidth), rtRect.topLeft.y);

    FntSetFont(fiCurrent);
}

void DrawNumbers (FormPtr pForm)
{
	DrawNumber();
}
