#include "StdInc.h"
#include "StringCtrl.h"

// Structs 
typedef struct _tagStringData
{
	MemHandle hString;
	UInt16 wFrameType;
	UInt16 wAlignment:2;
	UInt16 wClickable:1;
	UInt16 wGrayRect:1;
	UInt16 wUnused:12;
} StringData;
typedef StringData *StringDataPtr;

// Defines

// Forward Declarations
void StringCtrlDrawCtrl (FormPtr pForm, StringData *pStringData, RectangleType *pCtrlRect, Boolean bSelected);

void StringCtrlConstructor (FormPtr pForm, UInt16 wStringId, Char *pString, 
							UInt16 wFrameType, UInt16 wAlignment,
							Boolean bClickable, Boolean bGrayRect)
{
	StringDataPtr pStringData;
	MemHandle hStringData;

	hStringData = MemHandleNew(sizeof(StringData));
	if (hStringData)
	{
		pStringData = (StringDataPtr)MemHandleLock(hStringData);
		pStringData->hString = 0;
		pStringData->wAlignment = wAlignment;
		pStringData->wClickable = bClickable;
		pStringData->wFrameType = wFrameType;
		pStringData->wGrayRect = bGrayRect;
		MemHandleUnlock(hStringData);

		FrmSetGadgetData(pForm, FrmGetObjectIndex(pForm, wStringId), hStringData);
		StringCtrlSetString(pForm, wStringId, pString);
	}
}

void StringCtrlDestructor (FormPtr pForm, UInt16 wStringId)
{
	StringDataPtr pStringData;
	MemHandle hStringData;

	hStringData = FrmGetGadgetData(pForm, FrmGetObjectIndex(pForm, wStringId));
	if (hStringData)
	{
		pStringData = (StringDataPtr)MemHandleLock(hStringData);
		if (pStringData->hString)
			MemHandleFree(pStringData->hString);
		MemHandleUnlock(hStringData);
		MemHandleFree(hStringData);
	}
}

void StringCtrlSetUsable (FormPtr pForm, UInt16 wStringId, Boolean bUsable)
{
	GadgetPtr pGadget;

	pGadget = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wStringId));
	pGadget->attr.usable = bUsable;
	StringCtrlDraw(pForm, wStringId);
}

Boolean StringCtrlGetUsable (FormPtr pForm, UInt16 wStringId)
{
	GadgetPtr pGadget;

	pGadget = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, wStringId));
	return pGadget->attr.usable;
}

void StringCtrlDrawCtrl (FormPtr pForm, StringData *pStringData, RectangleType *pCtrlRect, Boolean bSelected)
{
	Boolean bVersion350 = IsRomVersion(VERSION_350);
	Char *pString;

	if (bVersion350 == true)
	{
		// Save the draw state.
		WinPushDrawState();

		// Draw the border.
		WinSetForeColor(UIColorGetTableEntryIndex(UIObjectFrame));
	}

	if (pStringData->wGrayRect)
	{
		WinDrawGrayRectangleFrame(pStringData->wFrameType, pCtrlRect);
	}
	else
	{
		WinDrawRectangleFrame(pStringData->wFrameType, pCtrlRect);
	}

	if (bVersion350 == true)
	{
		// Draw the inside.
		WinSetForeColor(UIColorGetTableEntryIndex(((bSelected == true) ? UIObjectSelectedFill : UIObjectFill)));
	}
	WinDrawRectangle(pCtrlRect, ((pStringData->wFrameType >> 8) & 0x00ff));

	// Draw the string.
	if (pStringData->hString != 0)
	{
		if (bVersion350 == true)
		{
			WinSetTextColor(UIColorGetTableEntryIndex(((bSelected == true) ? UIObjectSelectedForeground : UIObjectForeground)));
			WinSetBackColor(UIColorGetTableEntryIndex(((bSelected == true) ? UIObjectSelectedFill : UIObjectFill)));
		}

		pString = (Char*)MemHandleLock(pStringData->hString);
		DrawCharacters (pString, StrLen(pString), pCtrlRect, pStringData->wAlignment, true);
		MemHandleUnlock(pStringData->hString);
	}

	if (bVersion350 == true)
	{
		// Restore the draw state.
		WinPopDrawState();
	}
}

void StringCtrlDraw(FormPtr pForm, UInt16 wStringId)
{
	UInt16 wGadgetIndex;
	RectangleType vCtrlRect;
	GadgetPtr pGadget;
	StringDataPtr pStringData;
	MemHandle hStringData;

	wGadgetIndex = FrmGetObjectIndex(pForm, wStringId);
	FrmGetObjectBounds(pForm, wGadgetIndex, &vCtrlRect);

	pGadget = FrmGetObjectPtr(pForm, wGadgetIndex);
	if (pGadget->attr.usable)
	{
		StringDataPtr pStringData;
		MemHandle hStringData;

		hStringData = FrmGetGadgetData(pForm, wGadgetIndex);
		if (hStringData)
		{
			pStringData = (StringDataPtr)MemHandleLock(hStringData);
			if (IsRomVersion(VERSION_350))
			{
				StringCtrlDrawCtrl (pForm, pStringData, &vCtrlRect, false);
			}
			else
			{
				// Draw the text.
				if (pStringData->hString)
				{
					Char *pTemp;

					pTemp = (Char*)MemHandleLock(pStringData->hString);
					DrawCharacters (pTemp, StrLen(pTemp), &vCtrlRect, pStringData->wAlignment, true);
					MemHandleUnlock(pStringData->hString);
				}

				// Draw the frame
				if (pStringData->wGrayRect)
					WinDrawGrayRectangleFrame(pStringData->wFrameType, &vCtrlRect);
				else
					WinDrawRectangleFrame(pStringData->wFrameType, &vCtrlRect);
			}
			MemHandleUnlock(hStringData);
		}
	}
	else
	{
		InflateRect(&vCtrlRect, 1, 1);
		WinEraseRectangle(&vCtrlRect, 0);
	}
}

Int16 StringCtrlTap(FormPtr pForm, UInt16 wStringId, EventPtr pEvent)
{
	Boolean bVersion350 = IsRomVersion(VERSION_350);
	UInt16 wGadgetIndex;
	GadgetPtr pGadget;
	RectangleType vCtrlRect;
	StringDataPtr pStringData;
	MemHandle hStringData;
	Int16 iResult, iLastResult;
	Int16 x, y;
	Boolean bPenDown;
	UInt16 wFrameType;

	// Check for the click inside of the object.
	wGadgetIndex = FrmGetObjectIndex(pForm, wStringId);
	FrmGetObjectBounds(pForm, wGadgetIndex, &vCtrlRect);
	if (!RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vCtrlRect))
		return STRINGCTRL_NOTAP;

	// Check if disabled.
	pGadget = FrmGetObjectPtr(pForm, wGadgetIndex);
	if (!pGadget->attr.usable)
		return STRINGCTRL_NOTAP;

	// Check the clickable attribute.
	hStringData = FrmGetGadgetData(pForm, FrmGetObjectIndex(pForm, wStringId));
	pStringData = (StringDataPtr)MemHandleLock(hStringData);
	if (!pStringData->wClickable)
	{
		MemHandleUnlock(hStringData);
		return STRINGCTRL_NOCLICK;
	}

	iLastResult = STRINGCTRL_NOTAP;
	do 
	{
		PenGetPoint(&x, &y, &bPenDown);
		iResult = (RctPtInRectangle(x, y, &vCtrlRect)) ? STRINGCTRL_TAP : STRINGCTRL_NOTAP;
		if (iLastResult != iResult)
		{
			if (bVersion350 == true)
			{
				StringCtrlDrawCtrl (pForm, pStringData, &vCtrlRect, (iResult == STRINGCTRL_TAP));
			}
			else
			{
				WinInvertRectangle(&vCtrlRect, ((pStringData->wFrameType >> 8) & 0x00ff));
			}
			iLastResult = iResult;
		}
	} while (bPenDown);
	
	if (iResult != STRINGCTRL_NOTAP)
	{
		if (bVersion350 == true)
		{
			StringCtrlDrawCtrl(pForm, pStringData, &vCtrlRect, false);
		}
		else
		{
			WinInvertRectangle(&vCtrlRect, ((pStringData->wFrameType >> 8) & 0x00ff));
		}
		SndPlaySystemSound(sndClick);
	}
	MemHandleUnlock(hStringData);

	return iLastResult;
}

void StringCtrlSetString (FormPtr pForm, UInt16 wStringId, Char *pString)
{
	StringDataPtr pStringData;
	MemHandle hStringData;
	Char *pTemp;

	hStringData = FrmGetGadgetData(pForm, FrmGetObjectIndex(pForm, wStringId));
	if (hStringData)
	{
		pStringData = (StringDataPtr)MemHandleLock(hStringData);
		if (pStringData->hString)
			MemHandleFree(pStringData->hString);
		if (pString)
		{
			pStringData->hString = MemHandleNew(StrLen(pString) + 1);
			if (pStringData->hString)
			{
				pTemp = (Char*)MemHandleLock(pStringData->hString);
				StrCopy(pTemp, pString);
				MemHandleUnlock(pStringData->hString);
			}
		}
		else
		{
			pStringData->hString = 0;
		}
		MemHandleUnlock(hStringData);
	}
}

void StringCtrlGetString (FormPtr pForm, UInt16 wStringId, Char *pString)
{
	StringDataPtr pStringData;
	MemHandle hStringData;
	Char *pTemp;

	hStringData = FrmGetGadgetData(pForm, FrmGetObjectIndex(pForm, wStringId));
	if (hStringData)
	{
		pStringData = (StringDataPtr)MemHandleLock(hStringData);
		if (pStringData->hString)
		{
			pTemp = (Char*)MemHandleLock(pStringData->hString);
			StrCopy(pString, pTemp);
			MemHandleUnlock(pStringData->hString);
		}
		MemHandleUnlock(hStringData);
	}
}
